#!/usr/bin/env python
'''
Script to perform some simple management and get some information form bibtex files.

'''
import sys
import os
import optparse

import yapbib.biblist as biblist

################################################################################
# CUSTOMIZE THESE VARIABLES if needed
dumpfile= os.getenv('BIBDB',os.path.join(os.environ['HOME'],'texmf/bibtex/bib/bib.dmp'))
#*******************************************************************************
encoding='utf8'
  
def main():
  # CONFIGURACION ############################################################


  def get_strng_field(k):
    l= unicode(k,encoding=encoding).split(':')
    if len(l) == 1:  # argument was on the form 'search_string. To search in all fields
      ff= []
      ss= l[0]
    elif len(l) == 2:
      if l[0]=='': ss= '*' # Search all strings
      else:        ss= l[0]
      if l[1]=='':   ff= [] # Search in all fields
      else:          ff= l[1].split(':')
    return ss,ff

  #########################################################################################
  # Command line options
  #########################################################################################
  usage = """usage: %prog [options] [datafile1] [datafile2 ...]
  Ejemplo de uso:

  $> %prog --search=LastName1:author --search=LastName2:author --search=LastName3:author --startyear=2000 --endyear=2008 --filter-exclude=2006:year --filter-exclude=LastName4:author --sort=year,month,author --format=tex --output=salida.tex biblio1.bib biblio2.bib.bz2 biblio1.dmp biblio2.dmp.gz

Will get data from two bibtex files (biblio1.bib and biblio2.bib.bz2) and two dump files (biblio1.dmp and biblio2.dmp.gz) and retain all entries between 2000 and 2008 (except those of 2006) by authors LastName1,LastName2 and LastName3 but where LastName4 is not an author. The search is case insensitive. The output is written in latex form, ordered by key in ascending order, to the file salida.tex

  $> %prog - -o biblio.html
Will get the data from standard input in BibTeX format and output it in html form to the file biblio.html

                           ******** Working with pipes ********
  $> %prog -s LastName1:author biblio1.bib -f bib -o - | %prog -s LastName2:author biblio2.dmp - -o biblio.html

Will get the items with LastName1 as author from biblio1.bib and the results are taken as input to merge with items by LastName2 from database biblio2.dmp. The output is in html format to the file biblio.html

Note that two of the input files are compressed
  """
  parser = optparse.OptionParser(usage)

  parser.add_option("", "--list", action="store_true"
                    , help="List the database contents")

  parser.add_option("", "--sort"
                    , help="Sort the items according to the following fields, for instance to sort them accoding to year and then author we would use --sort=year,author. In the same example, to sort in reverse order we would use: --sort=year,author,reverse. DEFAULT: key.")

  parser.add_option("-s", "--search", action='append', type='string',
                    help='SEARCH is a (COLON separated) pair "string_to_search:fields". If the field is empty defaults to ALL. Fields may be more than one. In that case it can be written as "field1,field2,...". This option may be used more than once')

  parser.add_option("--year", default= None,
                    help="--year=y is a shortcut to '--start-year=y --end-year=y'")

  parser.add_option("-b", "--startyear", type='int', default=0
                    , help='Start Year')

  parser.add_option("-e", "--endyear", type='int', default=9999
                    , help='End Year')

  parser.add_option("-i", "--filter-include", action='append', type='string'
                    , help='Include all entries that verify the condition, given in the form string1:field1,field2,...  It may be used more than once and only entries that verify ALL conditions will be retained.')

  parser.add_option("-x", "--filter-exclude", action='append', type='string'
                    , help='Exclude all entries that verify the condition, given in the form string1:field1,field2,...  It may be used more than once and only entries that do not verify ANY condition will be retained.')

  parser.add_option("", "--keep-keys", action="store_true", default=False
                    , help="Keep the original cite key")

  parser.add_option("-I", "--case-sensitive", action="store_true", default=False
                    , help="Make the search case sensitive")

  parser.add_option("-o", "--output", default=None
                    , help="Output file. Use '-' for stdout (screen). DEFAULT: No output")

  parser.add_option("-f", "--format", default=None
                    , help="format of output, possible values are: short, full, bibtex, tex, html, xml   DEFAULT= short")

  parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False
                    , help="Give some informational messages.")

  parser.add_option("-d","--save-dump"
                    , help="Save (dump) the database IN INTERNAL FORM for faster access")

  (op, args) = parser.parse_args()

  if args == []: dbfiles= [dumpfile]
  else:          dbfiles= args
  modify_keys= not op.keep_keys
  available_formats= {'s':'short','f':'full','t':'latex','b':'bibtex','h':'html','x':'xml'}

  # Try to guess the format from the extension of the file
  if op.format == None:    # Guess a format
    if op.output == '-':
      formato='short'
    elif op.output != None:
      ext= os.path.splitext(op.output)[1][1]
      formato=available_formats.get(ext,'short')
  else:
    formato=available_formats.get(op.format[0].lower(),'short')

  #########################################################################################
  # Create the List object
  b= biblist.BibList()
  #########################################################################################
  # Read the database(s)
  if op.verbose:  print '# Loading database...'
  b= biblist.BibList()
  for fname in dbfiles:
    failed=False
    if '.dmp' in fname:
      try:    b.load(fname)
      except: failed=True
    elif '.bib' in fname or fname=='-':
      try: b.import_bibtex(fname, normalize=modify_keys)
      except: failed=True
    else: failed= True
    if op.verbose:  print '# %d new items read' %(len(b.ListItems))

    if failed: 
      mensaje= 'Database file %s not found or failed to load. Set the name as an option or set the environment variable BIBDB\n'%(fname)
      parser.error(mensaje)

  if op.sort != None:
    sortorder= op.sort.lower().split(',')
    if 'reverse' in sortorder:      reverse=True
    else: reverse= False
    if reverse: sortorder.remove('reverse')
  else:
    sortorder=[]
    reverse= False

  #######################################################################################
  # Do the required action(s)
  bout= biblist.BibList()
  items= b.sortedList[:]  # All items
  bout.abbrevDict.update(b.abbrevDict)

  if op.list:
    b.sort(sortorder,reverse)
    print '\n'.join(b.sortedList)
    return
  
  for k in  items:
    year= int(b.get_item(k).get_field('year',str(op.startyear)))
    if year >= op.startyear and year <= op.endyear:
      bout.add_item(b.get_item(k),k)

  if op.search!=None:
    items=[] # overwrite items from sort
    for cond in op.search:
      ss,ff= get_strng_field(cond)
      # search and append the results.
      items.extend(bout.search(findstr=ss,fields=ff, caseSens=op.case_sensitive))
    for it in bout.sortedList[:]:  # purge not found items
      if it not in items:
        bout.remove_item(it)
  
  if op.filter_exclude != None:
    items= [] 
    for cond in op.filter_exclude:
      ss,ff= get_strng_field(cond)
      items.extend(b.search(findstr=ss,fields=ff, caseSens=op.case_sensitive))
    for it in bout.sortedList[:]:    # purge found items
      if it in items:
        bout.remove_item(it)

  if op.filter_include != None:
    items= []
    cond= op.filter_include[0]
    ss,ff= get_strng_field(cond)
    items= b.search(findstr=ss,fields=ff, caseSens=op.case_sensitive)
    for cond in op.filter_include[1:]:
      ss,ff= get_strng_field(cond)
      its= b.search(findstr=ss,fields=ff, caseSens=op.case_sensitive)
      for c in items[:]:
        if c in items and c not in its:
          items.remove(c)
    for it in bout.sortedList[:]:    # purge not found items
      if it not in items:
        bout.remove_item(it)

  # First sort
  if op.sort != None:
    bout.sort(sortorder,reverse=reverse)

  if op.output != None:
    bout.output(op.output, formato, op.verbose)
  else:
    print '# %d items processed'%(len(bout.ListItems))

  if op.save_dump != None:
    if op.verbose:  print '# Saving database to %s...' %(op.save_dump)
    bout.dump(op.save_dump)

if __name__ == "__main__":
  main()


### Local Variables: 
### tab-width: 2
### END:



#!/usr/bin/env python
"""
Set of routines to parse bibtex data and return each entry as a dictionary
It is mainly intended as a helper file to the Class BibItem (see bibitem.py)
but can be used as a standalone script

USAGE:
  strings,db = parsefile(bibtexfile)

"""


import sys
import re,string
import codecs
import latex
latex.register()

import helper

reg_pages=re.compile(r'\W+')

def process_pages(pages):
  """ Returns a 2-tuple (firstpage,lastpage) from a string"""
  pp=reg_pages.split(pages)
  firstpage=pp[0]
  if len(pp)==2:
    lastpage=pp[1]
  else:
    lastpage=''
  return firstpage,lastpage
# def process_pages(pages):
#   """ Returns a 2-tuple (firstpage,lastpage) from a string"""
#   pp=pages.split('-')
#   firstpage=pp[0]
#   if len(pp)==2:
#     lastpage=pp[1]
#   else:
#     lastpage=''
#   return firstpage,lastpage

def bibtexauthor(data):
  """ Returns a list of authors where each author is a list of the form:
  [von, Last, First, Jr]
  """
  return map(helper.process_name,helper.removebraces(data).split(' and '))

def get_fields(strng, strict=False):
  """
  Returns a list with pairs (field, value) from strng
  If strict is True, it will only allow known fields, defined in helper.bibtexfields
  """ 

  comma_rex=re.compile(r'\s*[,]')
  ss=strng.strip()
  
  if not ss.endswith(','): # Add the last commma if missing
    ss+=','
    
  fields=[]

  while True:
    name,sep,ss= ss.partition('=')
    name=name.strip().lower()  # This should be enough if there is no error in the entry
    if len(name.split()) > 1:   # Help recover from errors. name should be only one word anyway
      name= name.split()[-1]
    ss=ss.strip()
    if sep == '': break  # We reached the end of the string

    if ss[0] == '{':    # The value is surrounded by '{}'
      s,e= helper.match_pair(ss)
      data= ss[s+1:e-1].strip()
    elif ss[0] == '"':  # The value is surrounded by '"'
      s= ss.find(r'"')
      e= ss.find(r'"',s+1)
      data= ss[s+1:e].strip()
    else: # It should be a number or something involving a string
      e= ss.find(',')
      data= ss[0:e].strip()
      if not data.isdigit(): # Then should be some string
        dd=data.split('#')  # Test for joined strings
        if len(dd) > 1:
          for n in range(len(dd)):
            dd[n]= dd[n].strip()
            dd[n]= dd[n].replace('{','"').replace('}','"')
            if dd[n][0] != '"':
              dd[n]='definitionofstring(%s) '%(dd[n])
          data='#'.join(dd)
        else:
          data='definitionofstring(%s) '%(data.strip())
    s=ss[e].find(',')
    ss=ss[s+e+1:]
# JF: Temporario, descomentar si hay problemas
#     if name=='title':
#       data=helper.capitalizestring(data)
#     else:
#       data=helper.removebraces(data)
    if not strict or name in helper.bibtexfields:
      fields.append((name,data))
  return fields


# Creates a (hopefully) unique key code
def create_entrycode(b={}):
  """
  Creates a 'hopefully unique' entry key from a bibtex item
  """
  len_aut=7  # Length of the author surname used
  try:
    aut= helper.capitalizestring('%s%s'%(b['author'][0][0],b['author'][0][1]))
  except:
    print b['author']
    print b['_code']
  aut=helper.oversimplify(aut)
  if len(aut) > len_aut:
    bibid=aut[:len_aut]
  else:
    bibid=aut.strip()

  bibid +=  b.get('year','')
  bibid += b.get('journal_abbrev','')

  if b['_type'] == 'mastersthesis':   bibid+= 'MT'
  elif b['_type'] == 'phdthesis':    bibid+= 'PHD'
  elif b['_type'] in ['book','incollection','proceedings','conference','misc','techreport']:
    if b.has_key('booktitle'):
      bibid+= helper.create_initials(b['booktitle'])
    elif b.has_key('series'):
      bibid+= helper.create_initials(b['series'])

    if b.has_key('title'):
      bibid+= '_'+helper.create_initials(b.get('title','').upper())[:3]

  if 'thesis' not in b['_type']:
    if b.has_key('firstpage'):   bibid+= 'p'+b['firstpage'].strip()
    elif b.has_key('volume'):    bibid+= 'v'+b['volume'].strip()
  return helper.oversimplify(bibid)


def replace_abbrevs(strs,bitem):
  """ Resolve all abbreviations found in the value fields of one entry"""
  b=bitem
  for f,v in b.iteritems():
    if helper.is_string_like(v):  b[f]= helper.replace_abbrevs(strs,v)
  return b

def parsefile(fname=None):
  fi= helper.openfile(fname)
  s= fi.read()
  db= parsedata(s)
  return db

def parsedata(data):
  """
  Parses a string with a bibtex database
  """
  # Regular expressions to use
  pub_rex = re.compile('\s?@(\w*)\s*[{\(]') # A '@' followed by any word and an opening
                                             # brace or parenthesis
  ########################################################################################
              #################### Reformat the string ####################
  ss= re.sub('\s+',' ',data).strip()

  # Find entries
  strings={}
  preamble=[]
  comment=[]
  tmpentries=[]
  entries={}

  while True:
    entry={}
    m= pub_rex.search(ss)

    if m == None:
      break

    if m.group(0)[-1]=='(':
      d= helper.match_pair(ss,pair=('[(]','[)]'),start=m.end()-1)
    else:
      d= helper.match_pair(ss,start=m.end()-1)

    if d != None:
      current= ss[m.start():d[1]-1]  # Currently analyzed entry
      st,entry= parseentry(current)
      if st != None:
        strings.update(st)
      if entry != None and entry != {}:
        entries[entry['_code']]= entry
      ss=ss[d[1]+1:].strip()

  return strings,entries

def parseentry(source):
  """
  Reads an item in bibtex form from a string 
  """
  try:
    source+' '
  except:
    raise TypeError
  # Transform Latex symbols and strip newlines and multiple spaces 
  
  source= source.decode('latex+utf8','ignore')
  source.replace('\n',' ')
  source= re.sub('\s+',' ',source)

  entry={}
  st= None
  s=source.partition('{')

  if s[1]=='':
    return None,None

  arttype= s[0].strip()[1:].lower()

  if arttype == 'string':
    # Split string name and definition, removing outer "comillas" and put them in a list
    name, defin = s[2].strip().split("=")
    defin= defin.replace('"','').strip()
    if defin.startswith('{'):
      defin=defin[1:-1]
    return {name.strip():defin.strip()},None

  elif arttype in helper.alltypes:
    # Then it is a publication that we want to keep
    p = re.match('([^,]+),', s[2] ) # Look for the key followed by a comma
    entry['_type']= arttype
    entry['_code']= p.group()[:-1]

    ff= get_fields(s[2][p.end():])
    for n,d in ff:
      if n == 'author' or n == 'editor':
        entry[n]= bibtexauthor(d)
      elif n== 'pages':
        entry['firstpage'],entry['lastpage']= process_pages(d)
      elif n == 'year':
        entry[n]= d.strip('.')
      else:
        entry[n]=d
    return None,entry

  elif arttype == 'comment' or arttype == 'preamble':
    # Do nothing (for now)
    return None,None
  else:
    return None,None


def test():
  if sys.argv[1:]:
    filepath= sys.argv[1]
  else:
    print "No input file"
    print "USAGE:  "+sys.argv[0]+ " FILE.bib\n\n  It will output the XML file: FILE.xml"
    sys.exit(2)

  strings,db = parsefile(filepath)
  print db
    
def main():
  test()

if __name__ == "__main__": main()




import re
import string
import sys

#        ################################ Variables  ################################
# List of all possible fields.
allfields = ('_type', 'address', 'author', 'booktitle', 'chapter', 'edition','_code',
'editor', 'howpublished', 'institution', 'journal', 'month', 'number', 'organization',
'pages', 'publisher', 'school','series', 'title', 'volume','year', 'note', 'code',
'url', 'crossref', 'annote', 'abstract', 'doi','journal_abbrev','date-added','date-modified', 'file')

bibtexfields = ('author', 'title', 'journal', 'year', 'volume','number', 'pages', 'month',
'booktitle', 'chapter','address', 'edition', 'abstract', 'doi',  'url', 'editor',
'howpublished','school', 'institution', 'organization', 'publisher', 'series',
'note', 'crossref', 'annote','file')

# Fields that MUST be present
minimalfields = ('_type','author')

# Fields that are output literally in bibtex form
textualfields = ['title', 'journal', 'year', 'volume','number', 'month', 'booktitle', 'chapter',
'address', 'edition', 'howpublished','school', 'institution', 'organization', 'publisher', 'series',
'note', 'crossref', 'issn','isbn', 'keywords' ,'annote', '_code', 'doi',  'url',  'abstract','file']

# Fields that should not be wrapped (must go complete in the same line)
nowrapfields= ['url','doi','isbn','issn','crossref']

# list of all reference types. CURRENTLY NOT USED
alltypes = ('article', 'book', 'booklet','conference', 'inbook', 'incollection', 'inproceedings', 'manual',
'mastersthesis', 'misc', 'phdthesis', 'proceedings', 'techreport', 'unpublished')



strings_month = [
  ('jan', 'January'),
  ('feb', 'February'),
  ('mar', 'March'),
  ('apr', 'April'),
  ('may', 'May'),
  ('jun', 'June'),
  ('jul', 'July'),
  ('aug', 'August'),
  ('sep', 'September'),
  ('oct',  'October'),
  ('nov', 'November'),
  ('dec', 'December'),
  ]

standard_abbrev= strings_month[:]

xml_tags= [  # xml/html entities
  ('&', '&amp;'),
  ('<', '&lt;'),
  ('>', '&gt;'),
]
# Accent tags are not used. They are handled via latex codec
accent_tags= [  # Accents and other latin symbols
  (r'\"a', "&#228;"),
  (r"\`a", "&#224;"),
  (r"\'a", "&#225;"),
  (r"\~a", "&#227;"),
  (r"\^a", "&#226;"),
  (r'\"e', "&#235;"),
  (r"\`e", "&#232;"),
  (r"\'e", "&#233;"),
  (r"\^e", "&#234;"),
  (r'\"\i', "&#239;"),
  (r"\`\i", "&#236;"),
  (r"\'\i", "&#237;"),
  (r"\^\i", "&#238;"),
  (r'\"i', "&#239;"),
  (r"\`i", "&#236;"),
  (r"\'i", "&#237;"),
  (r"\^i", "&#238;"),
  (r'\"o', "&#246;"),
  (r"\`o", "&#242;"),
  (r"\'o", "&#243;"),
  (r"\~o", "&#245;"),
  (r"\^o", "&#244;"),
  (r'\"u', "&#252;"),
  (r"\`", "&#249;"),
  (r"\'", "&#250;"),
  (r"\^", "&#251;"),
  (r'\"A', "&#196;"),
  (r"\`A", "&#192;"),
  (r"\'A", "&#193;"),
  (r"\~A", "&#195;"),
  (r"\^A", "&#194;"),
  (r'\"E', "&#203;"),
  (r"\`E", "&#200;"),
  (r"\'E", "&#201;"),
  (r"\^E", "&#202;"),
  (r'\"I', "&#207;"),
  (r"\`I", "&#204;"),
  (r"\'I", "&#205;"),
  (r"\^I", "&#206;"),
  (r'\"O', "&#214;"),
  (r"\`O", "&#210;"),
  (r"\'O", "&#211;"),
  (r"\~O", "&#213;"),
  (r"\^O", "&#212;"),
  (r'\"U', "&#220;"),
  (r"\`", "&#217;"),
  (r"\'", "&#218;"),
  (r"\^", "&#219;"),
  (r"\~n", "&#241;"),
  (r"\~N", "&#209;"),
  (r"\c c", "&#231;")
  ,(r"\c C", "&#199;")
  ,(r"\circ", "o")
  ]

other_tags=[
  ('([^\\\\])~','\g<1> ') # Remove tilde (used in LaTeX for spaces)
  ,(r'\\textbf{([^{]+)}',r'<b>\g<1></b>')
  ,(r'\\\\emph{([^{]+)}',r'<em>\g<1></em>')
  ,(r'\\textit{([^{]+)}',r'<i>\1</i>')
  ,(r'\[^ {]+{(.+)}','\g<1> ') # Remove other unknown commands
]

# Some journal data.
# DOI is useful when a journal uses a common (unique) root
# ISSN is unique but ads does not provides it. We list (when available) two issn: [print, online]
# Regular expression will give us a general (but not exact) matching strategy
journal_data=[
  {'regexp':r'Nuc\w*[\.]?\s*Inst\w*[\.]?\s*(and)?\s*Meth\w*[\.]?\s*(in)?\s*(Phys(ics)?[\.]?)?\s*(R(esearch)?[\.]?)?\s*(S(ection)?[\.]?)?\s*B|nimb', 'nombre':r'Nuclear Instruments and Methods in Physics Research B', 'abbrev':'NIMB', 'doi':'10.1016/j.nimb','issn':['0168-583X']},
  {'regexp':'Int(ernational)?[\.]?\s*J(our(nal)?)?[\.]?\s*(of)?\s*Q(uantum)?[\.]?\s*Ch(em(istry)?)?[\.]?', 'nombre':'International Journal of Quantum Chemistry', 'abbrev':'IJQC','doi':'10.1002/qua','issn':['1097-461X','0020-7608']},
  {'regexp':r'J(ournal)?[\.]?\s*(of)?\s*Phys(ics)?[\.]?\s*A(\s*:\s*Math(ematical)?\s*(and)?\s*(General)?)?|jpa','nombre': r'Journal of Physics A: Mathematical and General', 'abbrev':'JPA','doi':'10.1088/0305-4470','issn':['1751-8113','1751-8121']},
  {'regexp':r'J(ournal)?[\.]?\s*(of)?\s*Phys(ics)?[\.]?\s*B(\s*:\s*At(omic)?[\.]?\s*Mol(ecular)?[\.]?\s*(and)?\s*(Opt(ical)?)?\s*Phys(ics)?)?|jpb','nombre':r'Journal of Physics B: Atomic Molecular and Optical Physics', 'abbrev':'JPB','doi':'10.1088/0953-4075','issn':['0953-4075','1361-6455']},
  {'regexp':r'Ph\w*[\.]? Rev\w* A|pra','nombre': r'Physical Review A','abbrev': 'PRA', 'doi':'10.1103/PhysRevA','issn':['1050-2947','1094-1622']},
  {'regexp':r'Ph\w*[\.]? Rew* B|prb','nombre': r'Physical Review B','abbrev': 'PRB', 'doi':'10.1103/PhysRevB','issn':['1098-0121','1550-235x']},
  {'regexp':r'Ph\w*[\.]? Rew* C|prc','nombre': r'Physical Review C','abbrev': 'PRC', 'doi':'10.1103/PhysRevC','issn':['0556-2813','1089-490X']},
  {'regexp':r'Ph\w*[\.]? Rev\w*[\.]? L\w*[\.]?|prl','nombre': r'Physical Review Letters','abbrev': 'PRL', 'doi':'10.1103/PhysRevLett','issn':['0031-9007','1079-7114']},
  {'regexp':r'Rev\w*[\.]?\s*(of)?\s*Mod(ern)?\s*Phys(ics)?|rmp','nombre': r'Review of Modern Physics','abbrev': 'RMP', 'doi':'Not Available','issn':['0034-6861','1539-0756']},
  # 
  {'regexp':r'J\w*[\.]? Phy\w*[\.]? Conf\w*[\.]?|jcps','nombre': r"Journal of Physics Conference Series",'abbrev': 'JPCS', 'doi':'10.1088/1742-6596','issn':['1742-6588 ','1742-6596']},
  {'regexp':r'Comp\w*[\.]? Phys\w*[\.]? C\w*[\.]?|cpc','nombre': r"Computer Physics Communications",'abbrev': 'CPC', 'doi':'10.1016/j.cpc','issn':['0010-4655']},
  {'regexp':r'Phys\w*[\.]? Scrip\w*[\.]?','nombre': r"Physica Scripta Volume T",'abbrev': 'PS', 'doi':'10.1238/Physica.Topical','issn':[]},
  {'regexp':r'Rad\w*[\.]? Phy\w*[\.]? Chem\w*[\.]?|rpc','nombre': r"Radiation Physics and Chemistry",'abbrev': 'RPC', 'doi':'10.1016/j.radphyschem','issn':[]},
  {'regexp':r'F\w*[\.]? Bo\w*[\.]? Sys\w*[\.]?|fbs','nombre': r"Few-Body Systems",'abbrev': 'FBS', 'doi':'10.1007/s00601','issn':[]},
  {'regexp':r'Eur\w*[\.]? J\w*[\.]? Phys\w*[\.]?|ejp','nombre': r"European Journal of Physics",'abbrev': 'EJP', 'doi':'Not Available','issn':['0143-0807','1361-6404']},
  {'regexp':r'Phys\w*[\.]? Let\w*[\.]? A|pla', 'nombre':r"Physics Letters A",'abbrev': 'PLA', 'doi':'Not Available','issn':['0375-9601']},
  # 
  {'regexp':r'Am\w*[\.]? J\w*[\.]? P|ajp', 'nombre':r"American Journal of Physics",'abbrev': 'AJP', 'doi':'10.1119/','issn':['0002-9505']},
  {'regexp':r'J\w*[\.]? Chem\w*[\.]? Phys\w*[\.]?|jcp|JCP','nombre': r"Journal of Chemical Physics",'abbrev': 'JPC', 'doi':'Not Available','issn':['0021-9606','1089-7690']}
  ]

def is_string_like(obj):
  'Return True if *obj* looks like a string (from matplotlib)'
  if isinstance(obj, (str, unicode)): return True
  try: obj + ''
  except: return False
  return True

def isIterable(obj):
  try: iter(obj)
  except: return False
  return True

def is_writable_file_like(obj):
  '''return true if *obj* looks like a file object with a *write* method (from matplotlib)'''
  return hasattr(obj, 'write') and callable(obj.write)
def is_readable_file_like(obj): # No se si funciona
  '''return true if *obj* looks like a file object with a *read* method '''
  return hasattr(obj, 'read') and callable(obj.read)

def to_filehandle(fname, flag='r', return_opened=False):
  """ from matplotlib
  *fname* can be a filename or a file handle.  Support for gzipped
  files is automatic, if the filename ends in .gz.  *flag* is a
  read/write flag for :func:`file`
  """
  if is_string_like(fname):
    if fname.endswith('.gz'):
      import gzip
      fh = gzip.open(fname, flag)
    elif fname.endswith('.bz2'):
      import bz2
      fh = bz2.BZ2File(fname,flag)
    else:
      fh = file(fname, flag)
    opened = True
  elif hasattr(fname, 'seek'):
    fh = fname
    opened = False
  else:
    raise ValueError('fname must be a string or file handle')
  if return_opened:
    return fh, opened
  return fh


def openfile(fname=None,intent='r'):
  if fname == None or fname == '-' or fname == '':
    if intent=='r': fi= sys.stdin
    else:           fi= sys.stdout
    return fi

  return to_filehandle(fname,intent)


reg_simplify= re.compile('\W')
def oversimplify(strng):
  """
  (Over)simplify a string by converting to latex and then removing everything that does not belong to a word
  """
  s=strng.encode('latex')
  s= reg_simplify.sub('',s)
  return s

def closefile(fi):
  if fi != sys.stdin and fi != sys.stdout and fi != sys.stderr:
    fi.close()


reg_defstrng= re.compile('[{]*DEFINITIONOFSTRING[(](\w+)[)][{]*(\s*[#]*\s*["]*\s*\w*\s*["]*\s*)[}]*',re.I)
# reg_defstrng= re.compile('[{]*DEFINITIONOFSTRING[(](\w+)[)][}]*',re.I)

def replace_abbrevs(strs,sourcestrng):
  if 'DEFINITIONOFSTRING' in sourcestrng or 'definitionofstring' in sourcestrng:
    v=sourcestrng
    for abbrev,defin in strs.iteritems():
      v= v.replace('DEFINITIONOFSTRING(%s)'%(abbrev),defin)
      v= v.replace('DEFINITIONOFSTRING(%s)'%(abbrev).lower(),defin)
      v= v.replace('DEFINITIONOFSTRING(%s)'%(abbrev).upper(),defin)
      v= v.replace('definitionofstring(%s)'%(abbrev),defin)
      v= v.replace('definitionofstring(%s)'%(abbrev).lower(),defin)
      v= v.replace('definitionofstring(%s)'%(abbrev).upper(),defin)
      d= ' '.join([x.strip().strip('"') for x in v.split('#')])
#       if v.find('"') != -1 and 'Phys' in v:
#         print 'v',v
#         print 'd',d
    return  d
  return sourcestrng

def process_name(name):
  """
  Process one name and separate it in '[von, Last, First, Jr]'
  """
  def getnames_form3(a):
    """
    Case with two commas: the name is of the format
    von Last, Jr, First
    like in: von Hicks, III, Michael
    """
    full_last= a[0].strip()
    full_first=a[1].strip()
    junior= a[2]
    von,last= get_vonlast(full_last)
    return [von.strip(),last.strip(),full_first.strip(),junior.strip()]

  def getnames_form2(a):
    """
    Case with one comma: the name is of the format
    von Last, First
    like in: von Hicks, Michael
    """
    full_last= a[0].strip()
    full_first=a[1].strip()
    junior= ''
    von,last= get_vonlast(full_last)
    return [von.strip(),last.strip(),full_first.strip(),junior]

  def getnames_form1(a):
    """
    Case with NO commas: the name is of the format
    First von Last
    like in: Michael von Hicks
    """
    last= a[0].split(' ')
    nfn= 0
    for l in last:
      if l !="" and not l[0].islower():
        nfn+=1
      else:
        break
    if nfn == len(last):
      nfn= -1

    full_first= ' '.join(last[:nfn])
    full_first=full_first.replace('.',' ')
    full_last=  ' '.join(last[nfn:])
    junior=" "
    von,last= get_vonlast(full_last)
    return [von.strip(),last.strip(),full_first.strip(),junior.strip()]

  def get_vonlast(full_last):
    von=""
    last=""

    for l in full_last.split(' '):
      if len(l)>0 and l[0].islower():
        von += l.lower() + " "
      else:
        last += l + " "
    return von,last

  # Start the processing
  a=name.split(',')
  if len(a) == 3:
    fullname= getnames_form3(a)
  elif len(a) == 2:
    fullname= getnames_form2(a)
  elif len(a) == 1:
    fullname= getnames_form1(a)
  else:
    fullname=[]

  return fullname



def create_initials(strng):
  return ''.join(filter(lambda x:x.isupper(),map(lambda x:x[0],strng.split())))

def identify_some_journals(bibitem,known_journals=journal_data):

  # First try to identify the ISSN
  if bibitem.has_key('issn'):
    for j in known_journals:
      if bibitem['issn']== j['issn']:
        return j['nombre'],j['abbrev']

  # Then the DOI
  if bibitem.has_key('doi'):
    for j in known_journals:
      if bibitem['doi'].find(j['doi']) != -1:
        if j['issn'] != []:      bibitem['issn']=j['issn'][0]
        return j['nombre'],j['abbrev']

  # If DOI does not work, identify the journal with the regexp
  if bibitem.has_key('journal'):
    for j in known_journals:
      if re.search(j['regexp'],bibitem['journal']) != None:
        if j['issn'] != []:      bibitem['issn']=j['issn'][0]
        return j['nombre'],j['abbrev']

    # If it is not a know journal, get the abbreviation from the first letters of each word
    abbrev= create_initials(bibitem['journal'])
    return bibitem['journal'],abbrev
  else:
    # If nothing worked return emtpy strings
    return '',''


def handle_math(str,orden=0):
  """
  Convierte entre tex <--> html. 
  Si orden =0 => bib -> html 
  Si orden =1 => html -> bib
  """
  mathexp= ([ (re.compile(r'\^([^{]){',re.I)  ,r'<sup>\1</sup>')
             ,(re.compile(r'\^{([^{]+)}',re.I),r'<sup>\1</sup>')
             ,(re.compile(r'_([^{]+){',re.I)  ,r'<sub>\1</sub>')
             ,(re.compile(r'_{([^{]+)}',re.I) ,r'<sub>\1</sub>')
             ,(re.compile(r'\\mathrm{([^{]+)}',re.I) ,r'{\1}')
             ],
            [( re.compile(r'<sub>([^<]*)</sub>',re.I),r'$_{\1}$')
             ,(re.compile(r'<sup>([^<]*)</sup>',re.I),r'$^{\1}$')
             ])
#   mathmarker= ('<math>','</math>')
  mathmarker= ('','')


  if orden==0:
    p= re.compile(r'\$([^\$]+)\$')  # Find math substrings
    if p.search(str):
      ini=0
      linecontent=''
      iterator = p.finditer(str)
      for match in iterator:
        strmath= match.group()[1:-1]
        linecontent += str[ini:match.start()]
        for i,o in mathexp[orden]:
          strmath= re.sub(i,o,strmath)
        linecontent += mathmarker[0] + strmath + mathmarker[1]
        ini= match.end()
      linecontent += str[ini:]
    else:
      return str
  else:
    for i,o in mathexp[orden]:
      str= i.sub(o,str)
    linecontent= str

  return linecontent

# return the string parameter without braces
#
rembraces_rex = re.compile('[{}]')
def removebraces(str):
  return rembraces_rex.sub('',str) 


# data = title string
# @return the capitalized string (first letter is capitalized), rest are capitalized
# only if capitalized inside braces
capitalize_rex = re.compile('({\w*})')
def capitalizestring(data):
  ss_list = capitalize_rex.split(data)
  ss = ''
  count = 0
  for phrase in ss_list:
    check = phrase.lstrip()

	 # keep phrase's capitalization the same
    if check.find('{') == 0:
      ss+= removebraces(phrase)
    else:
      # first word --> capitalize first letter (after spaces)
      if count == 0:
        ss+= phrase.capitalize() 
#         ss+= check.capitalize() 
      else:
        ss+= phrase.lower() 
      count+= 1
  return ss



def match_pair(expr, pair=(r'{',r'}'),start=0):
  """ 
  Find the outermost pair enclosing a given expresion
  
  pair is a 2-tuple containing (begin, end) where both may be characters or strings
  for instance:
    pair= ('[',']')  or
    pair= ('if','end if') or
    pair= ('<div>','</div>') or ...
    
    """

  beg=pair[0]
  fin=pair[1]

  #   find first opening
  sstart= expr.find(beg,start)

  count= 0

  if beg == fin:
    eend= expr.find(fin,sstart+1)
    return sstart,eend

  p= re.compile('('+beg +'|' + fin+')', re.M)
  ps= re.compile(beg, re.M)

  iterator = p.finditer(expr,start)

  for match in iterator:
    if ps.match(match.group()):
      count+= 1
    else:
      count+= -1
        
    if count == 0:
      return sstart, match.end()

  return None

def no_outer_parens(filecontents):
  # JF: TODO We should check/change this routine Convierte los
  # parentesis a llaves cuando contienen el item completo. 
  # Do checking for open parens will convert to braces
  paren_split = re.split('([(){}])',filecontents)

  open_paren_count = 0
  open_type = False
  look_next = False

  # rebuild filecontents
  filecontents = ''

  at_rex = re.compile('@\w*')

  for phrase in paren_split:
    if look_next:
      if phrase == '(':
        phrase = '{'
        open_paren_count += 1
      else:
        open_type = False
      look_next = False

    if phrase == '(':
      open_paren_count += 1
    elif phrase == ')':
      open_paren_count -= 1
      if open_type and open_paren_count == 0:
        phrase = '}'
        open_type = False
    elif at_rex.search( phrase ):
      open_type = True
      look_next = True

    filecontents = filecontents + phrase
  return filecontents

    
def replace_tags(strng, what='All'):
  ww=what.lower()
  if ww == 'all' or ww == 'xml':
    # encode character entities
    for i,o in xml_tags:
      strng = strng.replace(i, o )

  if ww == 'all' or ww == 'accents':
    # latex-specific character replacements
    for i,o in accent_tags:
      strng = string.replace(strng, i, o )

  if ww == 'all' or ww == 'other':
    # Other LaTeX tags, handled by regexps
    for i,o in other_tags:
      strng= re.sub(i,o,strng)

  return strng

# def filter_author(biblist, filter_options):
#   delete=[]
#   for ident,item in biblist.iteritems():
#     valid = True
#     if item.has_key('author'):
#       aut = [ ' '.join(k) for k in item['author']]
#     else:
#       print ident, item
#       sys.exit(1)


# def filter_bib(biblist, filter_options):
#   delete=[]
#   for ident,item in biblist.iteritems():
#     valid = True
#     for k,v,cond in filter_options :
#       valid= item.has_key(k) and valid
#       if valid:
#         if type(item[k]) == list:
#           if k == 'author':
#             aut = [ ' '.join( s ) for s in item['author']]
#             data= ' '.join( aut )
#           else:
#             data= join( item[k] )
#         else:
#           data= item[k]
#         valid = valid and ((v in data.lower()) == cond)

#         if not valid:
#           delete.append(ident)
#           break
#       else:
#         delete.append(ident)
#         break

#   # Collect all matching and return them
#   bib={}
#   for ident in biblist.keys():
#     if ident not in delete:
#       bib[ident]= biblist[ident].copy()

#   return bib






#! /usr/bin/env python

import sys,os
import re
import codecs
import helper

ads_fields={'R': 'Bibliographic Code',
            'A': 'Author List',
            'a': 'Book Authors',
            'F': 'Author Affiliation',
            'J': 'Journal Name',
            'V': 'Journal Volume',
            'D': 'Publication Date',
            'P': 'First Page of Article',
            'L': 'Last Page of Article',
            'T': 'Title (required)',
            't': 'Original Language Title',
            'C': 'Abstract Copyright',
            'O': 'Object Name',
            'Q': 'Subject Category',
            'G': 'Origin',
            'S': 'Score from the ADS query (output only)',
            'E': 'Electronic Data Table',
            'I': 'Links to other information (output only)',
            'U': 'for Electronic Document',
            'K': 'Keywords',
            'M': 'Language (if not English)',
            'N': 'Not Documented!!!',
            'X': 'Comment',
            'W': 'Database (if submitting for more than one)',
            'Y': 'DOI',
            'B': 'Abstract Text',
            'Z': 'References'
            }

ads_literal_fields={
  # Fields that do not need post-procesing. They are just assigned to the dict.
  'R': '_code',
  'V': 'volume',
  'P': 'firstpage',
  'L': 'lastpage',
  'T': 'title',
  'U': 'url',
  'K': 'keywords',
  'B': 'abstract'
  }

reg_begin= re.compile('^[\s]*%R ',re.M)
reg_adstags=re.compile('^[\s]*%([RAaFJVDPLTtCOQGSEIUKMNXWYBZ]) ',re.M)

###########################################################################################
     ##############################  Parsing routines  ##############################
def parsedata(page):
  """
  Parse the results from ADS and store the list of references in a dictionary.
  """
  biblist={}

  page= helper.handle_math(page,1)  
  s= reg_begin.split(page)[1:] # The first item is empty, we take it out

  for paper in s:
    item= parseentry(paper)
    if item.has_key('author'):
      biblist[item['_code']]= dict(item)
  if biblist == {}: return None
  return biblist


def parseentry(paper):
  """
  Parses an ADS entry
  """
  def get_journal(t):
    reg_vol=re.compile('Vol',re.I)
    s=reg_vol.search(t)
    if s != None: t=t[0:s.start()]
    return t.strip().strip(',')

  def get_affil(t):
    reg_af= re.compile('[ ]?[A-D][A-Z](?=\()') # Regexp: Hasta un espacio, seguido de dos mayusculas, seguido de un parentesis (que no se consume)
    F= reg_af.split(t)
    if len(F) > 1: F=F[1:]
    return F

  def adsauthor(data):
    """ Returns a list of authors where each author is a list of the form:
    [von, Last, First, Jr]
    """
    return map(helper.process_name,data.split(';'))

  # Start the parsing of the entry
  Id=paper[0:paper.index('%')].strip('\n').strip()
  item={'_code':Id}

  campos= reg_adstags.split(paper)[1:]
  for i1 in range(0,len(campos),2):
    c,v=campos[i1],campos[i1+1]
    if c in ads_literal_fields.keys():
      item[ads_literal_fields[c]]=v.strip('\n')
    elif c == 'J':
      journal= get_journal(v).strip('\n')
      if 'thesis' in  journal.lower():
        item['_type']= 'phdthesis'
        item['school']= journal
      else:
        item['_type']='article'
        item['journal']= journal
    elif c == 'A':
      authors=adsauthor(v)
      item['author']= authors
    elif c == 'a':
      authors=adsauthor(v)
      item['author']= authors
      item['_type']='book'
    elif c == 'D':
      date=v.split('/')
      if len(date)==2:
        item['month']= helper.strings_month[int(date[0])-1][1]
        item['year']=date[1].strip()
      else:
        item['year']=v[-1].strip()
    elif c == 'Y':
      v= v.replace('doi: ','')
      v= v.replace('DOI: ','')
      item['doi']= re.sub('\s','',v.split(';')[0])
    elif c == 'F':
      item['affiliation']= get_affil(v)

  if item.has_key('affiliation') and len(item['affiliation']) == 1: 
    # Correct affiliations to "one for each author" even if it is the same for all
    item['affiliation']=len(item['author'])*item['affiliation']

  return item

def parsefile(fname,encoding='utf-8'):
  helper.openfile(fname)
  s= fname.read()
  db= parsedata(s)
#   db= parsedata(unicode(s,encoding=encoding))
  helper.closefile(fname)
  return db




##########################################################################################
##########################################################################################

def main():
  inputstring="""

%R 2008AmJPh..76.1146M
%T A velocity-dependent potential of a rigid body in a rotating frame
%A Moreno, G. A.; Barrachina, R. O.
%F AA(Departamento de F\xedsica J. J. Giambiagi, Facultad de Ciencias Exactas y 
Naturales, UBA, Ciudad Universitaria, Pabell\xf3n I, 1428 Buenos Aires, 
Argentina), AB(Centro At\xf3mico Bariloche and Instituto Balseiro, Comisi\xf3n 
Nacional de Energ\xeda At\xf3mica and Universidad Nacional de Cuyo, 8400 S. C. de 
Bariloche, R\xedo Negro, Argentina)
%J American Journal of Physics, Volume 76, Issue 12, pp. 1146-1149 (2008).
%V 76
%D 12/2008 
%P 1146
%L 1149
%K classical mechanics,  physics education,  rotating bodies
%G AIP
%C (c) 2008: American Institute of Physics
%I ABSTRACT: Abstract;
   EJOURNAL: Electronic On-line Article (HTML);
   REFERENCES: References in the Article;
   AR: Also-Read Articles;
%U http://adsabs.harvard.edu/abs/2008AmJPh..76.1146M
%B We derive a velocity-dependent potential for describing the dynamics of 
a rigid body in a rotating frame. We show that, as for one-particle 
systems, the different components of this potential can be associated 
with electromagnetic analogs. We provide some examples to demonstrate 
the feasibility of using the potential as an alternative description of 
rigid body problems.
%Y DOI: 10.1119/1.2982632

%R 2008JPhB...41n5204M
%T Transfer ionization and total electron emission for 100 keV amu<SUP>-1</SUP> 
He<SUP>2+</SUP> colliding on He and H<SUB>2</SUB>
%A Mart\xednez, S.; Bernardi, G.; Focke, P.; Su\xe1rez, S.; Fregenal, D. 
%F Centro At\xf3mico Bariloche and Instituto BalseiroComisi\xf3n Nacional de Energ\xeda 
At\xf3mica and Universidad Nacional de Cuyo, Argentina., 8400 S C de Bariloche, 
R\xedo Negro, Argentina <EMAIL>bernardi@cab.cnea.gov.ar</EMAIL>
%J Journal of Physics B: Atomic, Molecular, and Optical Physics, Volume 41, 
Issue 14, pp. 145204 (2008).
%V 41
%D 7/2008 
%P 5204
%G IOP
%I ABSTRACT: Abstract;
   EJOURNAL: Electronic On-line Article (HTML);
   REFERENCES: References in the Article;
   AR: Also-Read Articles;
%U http://adsabs.harvard.edu/abs/2008JPhB...41n5204M
%B We have measured electron emission for transfer ionization (TI) and 
total electron emission (TEE, all emission processes) for 100 keV 
amu<SUP>-1</SUP> He<SUP>2+</SUP> on He and H<SUB>2</SUB> targets. Double 
differential cross sections have been obtained for emission angles 
\u03b8 = 0\xb0, 20\xb0 and 45\xb0, and electron energies ranging 
from 2 to 300 eV. Pure ionization, mainly due to single ionization, 
dominates the low-energy electron emission. The main observed structure 
in the electron spectra, a cusp centred at \u03b8 = 0\xb0 and at a 
speed equal to that of the incident projectile, presents an asymmetric 
shape. This is in contrast to the symmetric shape observed by us at 25 
keV amu<SUP>-1</SUP> for the same collision systems, suggesting a change 
in the cusp formation mechanism for TI within this energy range.
%Y DOI: 10.1088/0953-4075/41/14/145204

"""
  
  b= parsedata(inputstring)
  print '%d ADS items parsed\n'%(len(b.keys()))
  for v in b.itervalues():
    s=str(v)
    print s.encode('utf8'),'\n'
################################################################################
  c= parsefile('ejemplos/ejemplo1.ads')
  print '%d ADS items parsed\n'%(len(c.keys()))
  for v in c.itervalues():
    s=str(v)
    print s.encode('utf8'),'\n'
  
if __name__ == "__main__":
  main()


### Local Variables: 
### tab-width: 2
### END:



#! /usr/bin/env python
# Small module to query the Harvard Database at http://adsabs.harvard.edu
# It creates the query and returns the 
# import sys,os

import urllib2
import socket
import re

# Modelo de pedido
# http://adsabs.harvard.edu/cgi-bin/nph-abs_connect?db_key=AST&db_key=PHY&db_key=PRE&qform=PHY&arxiv_sel=astro-ph&arxiv_sel=cond-mat&arxiv_sel=cs&arxiv_sel=gr-qc&arxiv_sel=hep-ex&arxiv_sel=hep-lat&arxiv_sel=hep-ph&arxiv_sel=hep-th&arxiv_sel=math&arxiv_sel=math-ph&arxiv_sel=nlin&arxiv_sel=nucl-ex&arxiv_sel=nucl-th&arxiv_sel=physics&arxiv_sel=quant-ph&arxiv_sel=q-bio&aut_logic=OR&author=Fiol%2CJ&ned_query=YES&sim_query=YES&start_mon=&start_year=&end_mon=&end_year=&ttl_logic=OR&title=&txt_logic=OR&text=&nr_to_return=200&start_nr=1&article_sel=YES&jou_pick=NO&ref_stems=&data_and=ALL&group_and=ALL&start_entry_day=&start_entry_mon=&start_entry_year=&end_entry_day=&end_entry_mon=&end_entry_year=&min_score=&sort=SCORE&data_type=BIBTEX&aut_syn=YES&ttl_syn=YES&txt_syn=YES&aut_wt=1.0&ttl_wt=0.3&txt_wt=3.0&aut_wgt=YES&obj_wgt=YES&ttl_wgt=YES&txt_wgt=YES&ttl_sco=YES&txt_sco=YES&version=1
# http://adsabs.harvard.edu/cgi-bin/nph-abs_connect?db_key=AST&db_key=PHY&db_key=PRE&qform=PHY&arxiv_sel=astro-ph&arxiv_sel=cond-mat&arxiv_sel=cs&arxiv_sel=gr-qc&arxiv_sel=hep-ex&arxiv_sel=hep-lat&arxiv_sel=hep-ph&arxiv_sel=hep-th&arxiv_sel=math&arxiv_sel=math-ph&arxiv_sel=nlin&arxiv_sel=nucl-ex&arxiv_sel=nucl-th&arxiv_sel=physics&arxiv_sel=quant-ph&arxiv_sel=q-bio&aut_xct=YES&aut_logic=OR&author=Fiol%2CJ&ned_query=YES&sim_query=YES&start_mon=&start_year=2008&end_mon=&end_year=2008&ttl_logic=OR&title=&txt_logic=OR&text=&nr_to_return=200&start_nr=1&jou_pick=ALL&ref_stems=&data_and=ALL&group_and=ALL&start_entry_day=&start_entry_mon=&start_entry_year=&end_entry_day=&end_entry_mon=&end_entry_year=&min_score=&sort=NDATE&data_type=PORTABLE&aut_syn=YES&ttl_syn=YES&txt_syn=YES&aut_wt=1.0&ttl_wt=0.3&txt_wt=3.0&aut_wgt=YES&obj_wgt=YES&ttl_wgt=YES&txt_wgt=YES&ttl_sco=YES&txt_sco=YES&version=1
# http://adsabs.harvard.edu/cgi-bin/nph-abs_connect?db_key=AST&db_key=PHY&db_key=PRE&qform=PHY&arxiv_sel=astro-ph&arxiv_sel=cond-mat&arxiv_sel=cs&arxiv_sel=gr-qc&arxiv_sel=hep-ex&arxiv_sel=hep-lat&arxiv_sel=hep-ph&arxiv_sel=hep-th&arxiv_sel=math&arxiv_sel=math-ph&arxiv_sel=nlin&arxiv_sel=nucl-ex&arxiv_sel=nucl-th&arxiv_sel=physics&arxiv_sel=quant-ph&arxiv_sel=q-bio&aut_xct=YES&aut_logic=OR&author=Fiol%2CJ&ned_query=YES&sim_query=YES&start_mon=&start_year=2008&end_mon=&end_year=2008&ttl_logic=OR&title=&txt_logic=OR&text=&nr_to_return=200&start_nr=1&jou_pick=ALL&ref_stems=&data_and=ALL&group_and=ALL&start_entry_day=&start_entry_mon=&start_entry_year=&end_entry_day=&end_entry_mon=&end_entry_year=&min_score=&sort=NDATE&data_type=PORTABLE&aut_syn=YES&ttl_syn=YES&txt_syn=YES&aut_wt=1.0&ttl_wt=0.3&txt_wt=3.0&aut_wgt=YES&obj_wgt=YES&ttl_wgt=YES&txt_wgt=YES&ttl_sco=YES&txt_sco=YES&version=1

# allowed_options=['ned_query', 'ttl_wgt', 'text', 'ttl_syn', 'txt_logic', 'qform', 'start_nr', 'article_sel', 'aut_xct', 'ref_stems', 'start_entry_year', 'end_entry_year', 'group_and', 'txt_wgt', 'min_score', 'txt_wt', 'aut_wgt', 'author', 'ttl_wt', 'end_year', 'txt_sco', 'ttl_sco', 'end_entry_day', 'aut_wt', 'version', 'end_entry_mon', 'aut_syn', 'sort', 'start_year', 'nr_to_return', 'data_type', 'obj_wgt', 'sim_query', 'aut_logic', 'start_entry_mon', 'db_key', 'jou_pick', 'ttl_logic', 'arxiv_sel', 'data_and', 'end_mon', 'start_mon', 'start_entry_day', 'txt_syn', 'title']


all_param= {  # All parameters
'author':'list of semicolon separated authornames as last, f',
'object':'list of semicolon separated object names',
'keyword':'list of semicolon separated keywords',
'start_mon':'starting month as integer (Jan == 1, Dec == 12)',
'start_year':'starting year as integer (4 digits)',
'end_mon':'ending month as integer (Jan == 1, Dec == 12)',
'end_year':'ending year as integer (4 digits)',
'start_entry_day':'start entry day of month as integer',
'start_entry_mon':'start entry month as integer',
'start_entry_year':'start entry year as integer',
'end_entry_day':'start entry day of month as integer',
'end_entry_mon':'start entry month as integer',
'end_entry_year':'start entry year as integer',
'title':'title words, any non-alpha-numeric character separates',
'text':'abstract words, any non-alpha-numeric character separates',
'fulltext':'OCRd fulltext, any non-alpha-numeric character separates',
'affiliation':'affiliation words, any non-alpha-numeric character separates',
'bibcode':'bibcode for partial bibcode search. If a bibcode is specified no other search will be done',
'nr_to_return':'how many abstracts to return (default is 50, max 500)',
'start_nr':'where to start returning in list of retrieved abstracts default is 1',
'aut_wt':'floating point weight for author search, default: 1.0',
'obj_wt':'floating point weight for object search, default: 1.0',
'kwd_wt':'floating point weight for keyword search, default: 1.0',
'ttl_wt':'floating point weight for title search, default: 0.3',
'txt_wt':'floating point weight for text search, default: 3.0',
'full_wt':'floating point weight for full search, default: 3.0',
'aff_wt':'floating point weight for affiliation search, default: 1.0',
'aut_syn':'author synonym replacement. aut_syn="YES" turns it on (default is on)',
'ttl_syn':'title synonym replacement. ttl_syn="YES" turns it on (default is on)',
'txt_syn':'text synonym replacement. txt_syn="YES" turns it on (default is on)',
'full_syn':'full text synonym replacement. full_syn="YES" turns it on (default is on)',
'aff_syn':'affiliation synonym replacement. aff_syn="YES" turns it on (default is on)',
'aut_wgt':'authors used for weighting. aut_wgt="YES" turns it on (default is on)',
'obj_wgt':'objects used for weighting. obj_wgt="YES" turns it on (default is on)',
'kwd_wgt':'keywords used for weighting. kwd_wgt="YES" turns it on (default is on)',
'ttl_wgt':'title used for weighting. ttl_wgt="YES" turns it on (default is on)',
'txt_wgt':'text used for weighting. txt_wgt="YES" turns it on (default is on)',
'full_wgt':'full text used for weighting. full_wgt="YES" turns it on (default is on)',
'aff_wgt':'affiliation used for weighting. aff_wgt="YES" turns it on (default is on)',
'aut_sco':'authors weighted scoring. aut_sco="YES" turns it on (default is off)',
'kwd_sco':'keywords weighted scoring. kwd_sco="YES" turns it on (default is off)',
'ttl_sco':'title weighted scoring. ttl_sco="YES" turns it on (default is on)',
'txt_sco':'text weighted scoring. txt_sco="YES" turns it on (default is on)',
'full_sco':'text weighted scoring. full_sco="YES" turns it on (default is on)',
'aff_sco':'affiliation weighted scoring. aff_sco="YES" turns it on (default is off)',
'aut_req':'authors required for results. aut_req="YES" turns it on (default is off)',
'obj_req':'objects required for results. obj_req="YES" turns it on (default is off)',
'kwd_req':'keywords required for results. kwd_req="YES" turns it on (default is off)',
'ttl_req':'title required for results. ttl_req="YES" turns it on (default is off)',
'txt_req':'text required for results. txt_req="YES" turns it on (default is off)',
'full_req':'text required for results. full_req="YES" turns it on (default is off)',
'aff_req':'affiliation required for results. aff_req="YES" turns it on (default is off)',
'aut_logic':'obj_logic',
'kwd_logic':'ttl_logic',
'txt_logic':'full_logic',
'aff_logic':'Combination logic: "AND" combine with AND,"OR" combine with OR (default),"SIMPLE" simple logic (use +, -),"BOOL" full boolean logic, "FULLMATCH" do AND query in the selected field and calculate the score according to how many words in the field of the selected reference were matched by the query',
'return_req':	'requested return: return_req="result" : return results (default), return_req="form" : return new query form, return_req="no_params": return results, set default parameters, dont display params',
'db_key':	'which database to query: db_key="AST" : Astronomy(default); "PRE": arXiv e-prints; "PHY": Physics, "GEN": General, CFA: CfA Preprints',
'atcl_only':'select only OCR pages from articles',
'jou_pick':'specify which journals to select: "ALL" : return all journals (default), "NO" : return only refereed journals, "EXCL" : return only non-refereed journals',
'ref_stems':'list of comma-separated ADS bibstems to return, e.g. ref_stems="ApJ..,AJ..."',
'min_score':'minimum score of returned abstracts (floating point, default 0.0)',
'data_link':'return only entries with data., data_link="YES" turns it on, default is off',
'abstract':'return only entries with abstracts. "YES" turns it on, default is off',
'alt_abs':'return only entries with alternate abstracts. "YES" turns it on, default is off',
'aut_note':'return only entries with author notes. "YES" turns it on, default is off',
'article':'return only entries with articles. "YES" turns it on, default is off',
'article_link':'return only entries with electronic articles."YES" turns it on, default is off',
'simb_obj':'return only entries with simbad objects."YES" turns it on, default is off',
'ned_obj':'return only entries with ned objects."YES" turns it on, default is off',
'gpndb_obj':'return only entries with gpndb objects."YES" turns it on, default is off',
'lib_link':'return only entries with library links."YES" turns it on, default is off',
'data_and':'return only entries with all selected data available."ALL": no selection, return all refs (default), "NO" : return entries with AT LEAST ONE of the data items selected with the above flags"YES": return only entries that have ALL links selected with the above flags',
'version':'version number for the query form',
'data_type':'data type to return, "HTML" return regular list (default),"PORTABLE" return portable tagged format, "PLAINTEXT" return plain text,"BIBTEX" return bibtex format,"ENDNOTE" return ENDNOTE format,"DUBLINCORE" return DUBLINCORE format,"XML" return XML format,"SHORT_XML" return short XML format (no abstract),"VOTABLE" return VOTable format,"RSS" return RSS format',
'mail_link':'return only entries with mailorder."YES" turns it on, default is off',
'toc_link':'return only entries with tocorder."YES" turns it on, default is off',
'pds_link':'return only entries with pds data."YES" turns it on, default is off',
'multimedia_link':'return only entries with multimedia data."YES" turns it on, default is off',
'spires_link':'return only entries with spires data."YES" turns it on, default is off',
'group_and':'return only entries from all selected groups."ALL":no selection (default)"NO" :return entries that are in at least one grp, "YES":return only entries from ALL groups selected with group_bits',
'group_sel':'which group to select, e.g. group_sel="Chandra,HST"',
'ref_link':'return only entries with reference links."YES" turns it on, default is off',
'citation_link':'return only entries with citation links."YES" turns it on, default is off',
'gif_link':'return only entries with scanned articles links.',
'open_link':'return only entries with open access.',
'aut_xct':'exact author search. aut_xct="YES" turns it on',
'lpi_query':'lpi_query="YES" query for LPI objects, default is off',
'sim_query':'sim_query="YES" query for SIMBAD objects, default is on',
'ned_query':'ned_query="YES" query for NED objects, default is on',
'iau_query':'iau_query="YES" query for IAU objects, default is off',
'sort':'sort options:"SCORE": sort by score, "AUTHOR": sort by first author,"NDATE": sort by date (most recent first,"ODATE": sort by date (oldest first),"BIBCODE": sort by bibcode,"ENTRY": sort by entry date in the database,"PAGE": sort by page number,"RPAGE": reverse sort by page number,"CITATIONS": sort by citation count (replaces score with number of citations), "NORMCITATIONS": sort by normalized citation count, (replaces score with number of normalized citations), "AUTHOR_CNT": sort by author count',
'query_type':'what to return: query_type=PAPERS returns regular records (default), CITES returns citations to selected records, REFS returns references in selected records, ALSOREADS returns also-reads in selected records',
'return_fmt':'return format: return_fmt="LONG": return full abstract, "SHORT": return short listing (default)',
'type':'where to return the data (screen, file, printer, etc)',
'defaultset':'use default settings (same as ret_req=no_params but displays query parameters on short form)',
'charset':'character set for text output',
'year':'year field for bibcode matching',
'bibstem':'bibstem field for bibcode matching',
'volume':'volume field for bibcode matching',
'page':'page field for bibcode matching',
'associated_link':'return only entries with associated articles. associated_link="YES" turns it on, default is off',
'ar_link':'return only entries with AR links. ar_link="YES" turns it on, default is off',
'tables':'return results with table formatting (overrides pref.)',
'email_ret':'email_ret="YES": return query result via email',
'exclude':'exclude=bibcode1[,bibcode2...]: exclude specified bibcodes from results list',
'include':'include=bibcode1[,bibcode2...]: include specified bibcodes in results list',
'selectfrom':'selectfrom=bibcode1[,bibcode2...]: include only bibcodes from specified bibcode list',
'RA':'Right ascension for cone search',
'DEC':'Declination for cone search',
'SR':'Search radius for cone search (default is 10 arcmin)',
'method':'form method of query form: GET or POST',
'nfeedback':'number of records to use in feedback queries',
'doi':'DOI',
'preprint_link':'return only entries with preprint data.',
'preprint_link':'"YES" turns it on, default is off',
'refstr':'reference string to resolve',
'mimetype':'mimetype of returned page (default depends on data_type)',
'blog_link':'return only entries with blog links',
'qsearch':'quick search field, can be author or text',
'arxiv_sel':'which arxiv categories to select',
'article_sel':'select only articles (not catalogs, abstracts, etc)',
'adsobj_query':'search object names in abstract text',
}


param_relevantes= { #More relevant parameters
'author':'list of semicolon separated authornames as last, f',
'aut_xct':'exact author search. aut_xct="YES" turns it on',
'keyword':'list of semicolon separated keywords',
'start_mon':'starting month as integer (Jan == 1, Dec == 12)',
'start_year':'starting year as integer (4 digits)',
'end_mon':'ending month as integer (Jan == 1, Dec == 12)',
'end_year':'ending year as integer (4 digits)',
'title':'title words, any non-alpha-numeric character separates',
'affiliation':'affiliation words, any non-alpha-numeric character separates',
'text':'abstract words, any non-alpha-numeric character separates',
'fulltext':'OCRd fulltext, any non-alpha-numeric character separates',
'charset':'character set for text output',
'bibcode':'bibcode for partial bibcode search. If a bibcode is specified no other search will be done',
'db_key':	'which database to query: db_key="AST" : Astronomy(default); "PRE": arXiv e-prints; "PHY": Physics, "GEN": General, CFA: CfA Preprints',
'nr_to_return':'how many abstracts to return (default is 50, max 500)',
'start_nr':'where to start returning in list of retrieved abstracts default is 1',
'aut_logic':'Combination logic: "AND" combine with AND,"OR" combine with OR (default),"SIMPLE" simple logic (use +, -),"BOOL" full boolean logic, "FULLMATCH" do AND query in the selected field and calculate the score according to how many words in the field of the selected reference were matched by the query',
'jou_pick':'specify which journals to select: "ALL" : return all journals (default), "NO" : return only refereed journals, "EXCL" : return only non-refereed journals',
'doi':'DOI',
'ref_stems':'list of comma-separated ADS bibstems to return, e.g. ref_stems="ApJ..,AJ..."',
}

def isStringLike(obj):
  try: obj + ''
  except: return False
  else: return True
def isIterable(obj):
  try: iter(obj)
  except: return False
  else: return True


journal_ads_abbrev=[
  (r'\pra','Physical Review A'),
  (r'\prb','Physical Review B')
]

ads_fields={'R': 'Bibliographic Code',
            'A': 'Author List',
            'a': 'Book Authors',
            'F': 'Author Affiliation',
            'J': 'Journal Name',
            'V': 'Journal Volume',
            'D': 'Publication Date',
            'P': 'First Page of Article',
            'L': 'Last Page of Article',
            'T': 'Title (required)',
            't': 'Original Language Title',
            'C': 'Abstract Copyright',
            'O': 'Object Name',
            'Q': 'Subject Category',
            'G': 'Origin',
            'S': 'Score from the ADS query (output only)',
            'E': 'Electronic Data Table',
            'I': 'Links to other information (output only)',
            'U': 'for Electronic Document',
            'K': 'Keywords',
            'M': 'Language (if not English)',
            'N': 'Not Documented!!!',
            'X': 'Comment',
            'W': 'Database (if submitting for more than one)',
            'Y': 'DOI',
            'B': 'Abstract Text',
            'Z': 'References'
            }

ads_literal_fields={
  # Fields that do not need post-procesing. They are just assigned to the dict.
  'R': '_code',
  'V': 'volume',
  'P': 'firstpage',
  'L': 'lastpage',
  'T': 'title',
  'U': 'url',
  'K': 'keywords',
  'B': 'abstract'
  }

class AdsQuery(object):
  """Class that creates a query line and allows to perform the query to ads
  """

  url= "http://adsabs.harvard.edu/cgi-bin/nph-abs_connect?"
  std_opt=dict(
    db_key=["PHY","GEN"],
    data_type="PORTABLE",
    charset='utf-8',
    return_fmt="LONG",
    )

  def __init__(self, connection={}, options={}):
    self.opt= self.std_opt
    self.set_options(options)
    self.set_connection_options(connection)
    
  def set_options(self,options):
    if options.has_key('verbose'):
      self.verbose= options['verbose']
      del options['verbose']
    else:
      self.verbose= 0
    # We could check that the options are licit. May be we'll do later
    for k,v in options.iteritems():
      self.opt[k]= v

  def set_connection_options(self,connection):
    """
    Set Connection options, they are:
    url:         url of the database to query 
    http_proxy:  Proxy to use
    timeout:     time after which we will desist if nothing happens ;)
    """
    timeout= connection.get('timeout','90')
    # Create a timeout for all sockets (including the connections made by urrlib2)
    socket.setdefaulttimeout(float(timeout))

    if connection.has_key('url'):
      self.url= connection['url']

    if connection.has_key('http_proxy'):
      self.proxy= connection['http_proxy']
      if self.verbose > 1:
        print '# Using proxy', self.proxy
      # Create a proxy handler 
      proxy_handler = urllib2.ProxyHandler({'http':  self.proxy})
      # Replace with it the default proxy (from environment)
      opener = urllib2.build_opener(proxy_handler)
      # ...and install it globally so it can be used with urlopen.
      urllib2.install_opener(opener)

  def query(self):
    paper_req=self.__create_query__()
    if self.verbose > 0:
      print '# Query Command line:\n# ',paper_req,'\n'
    try:
      response = urllib2.urlopen(paper_req.encode(self.opt['charset']))
    except urllib2.URLError, e:
      if hasattr(e, 'reason'):
        return -2,'We failed to reach a server.\nReason: '+str(e.reason)
      elif hasattr(e, 'code'):
        return -1,'The server couldn\'t fulfill the request.\n' + 'Error code: ' + str(e.code)
    else:
      the_page= unicode(response.read(),encoding=self.opt['charset'])
      the_page= ''.join(the_page.splitlines(True)[4:])
      nabstracts= the_page.count('%R')
    return nabstracts, the_page

  def set_charset(encoding):
    self.charset=encoding

  def add_database(db):
    self.db_key.append(db)

  def del_database(db):
    try:
      self.db_key.remove(db)
    except:
      pass

  def __create_query__(self):
    paper_req=self.url
    for k,v in self.opt.iteritems():
      if isStringLike(v):
        op='%s%s%s%s' % (k,'=',v,'&')
      elif isIterable(v):
        op='%s%s%s%s' % (k,'=',';'.join(v),'&')
      else:
        op=''
        print '# Bad type for option: ', k, '   value:',v
      paper_req+= op
    paper_req=paper_req.replace(' ','')
    return paper_req[:-1]

def main():
  import re
#   reg_n_abstracts=re.compile('Retrieved ([0-9]+) abstracts')
  my_options={'author':('Fiol,J','Fuhr,J','Fregenal,D'),'start_year':'2008','end_year':'2008','verbose':2}

  Query= AdsQuery(options=my_options)
  status,page= Query.query()

  print page
  if status >= 0:
    print '%s\nFound %s entries\n%s'%(80*'*',status,80*'*')

if __name__ == "__main__":
  main()


### Local Variables: 
### tab-width: 2
### END:



#!/usr/bin/env python
'''
Class storing a bibliography

A Class that to store a list of references (list of papers, books, manuals, ...)
It is based in dictionaries, with a unique self-generated key

Example of use:
  
import yapbib.biblist as biblist
b=biblist.BibList()
b.import_bibtex("mybib.bib")
items= b.List() # Shows the keys of all entries
it= b.get_item(items[0]) # Get first item 
# (Alternatively, to get the first item you can use 
it= b.get_items()[0] 
it.get_fields() # Show all fields for item
it.preview()    # Show a preview (brief info)
bib= it.to_bibtex() # get item in BibTeX form
tex= it.to_latex() # get item in LaTeX form
html= it.to_html() # get item in html form
print it  # print full information on the item
print unicode(it) # Use this if it has non-ascii characters

'''
import sys
import os

import textwrap

import bibitem
import helper
import latex
latex.register()

import cPickle as pickle

# Index used for internally for each part of a name
# (A_VON, A_LAST, A_JR, A_FIRST)= range(4)

class BibList(dict):
  '''Class storing a bibliography  (list of papers, books, manuals, ...)'''
  def __init__(self):
    self.ListItems= []
    self.abbrevDict= dict(helper.standard_abbrev)
    self.sortorder=['_type','key'] # Fields used to sort the list
    self.sortedList=[]   # Sorted version of ListItems
    self.issorted=False # Keeps tracks whether the List is sorted
    self.reverse=False
    self.keepAbbrevs=True
    self.encoding='utf-8'
    self.bib={}

  def update(self,blist):
    try:
      self.bib.update(blist.bib)  # blist es un objeto del tipo BibList
      self.set_properties_from(blist)
    except:
      try: self.bib.update(blist)  # blist es solo un diccionario
      except:  raise TypeError, 'Argument incorrect type, must be BibList object'

  def set_properties_from(self,blist):
    """
    Try to get properties from other biblist object
    """
    try:
      self.ListItems += blist.ListItems
      self.ListItems= list(set(self.ListItems))
#       for k in blist.ListItems:
#         if k not in self.ListItems:  self.ListItems.append(k)
    except:  pass
    try:     self.abbrevDict.update(blist.abbrevDict)
    except:  pass
    if self.issorted: self.sort()
    else: self.sortedList = self.ListItems[:]

  def __str__(self):
    s=''
    if not self.issorted:      self.sortedList= self.sort()
    for l in self.sortedList:
      s+= '%s\n' %(self.get_item(l).__str__())
    return s.encode(self.encoding,'ignore')

  def __repr__(self):
    s= u'bib= %s\n'%(str(self.bib).decode(self.encoding,'ignore'))
    s+= u'abbrevDict= %s\n' %(self.abbrevDict)
    s+= u'ListItems= %s\n' %(self.ListItems)
    s+= u'sortedList= %s\n' %(self.sortedList)
    s+= u'issorted= %s\n' %(self.issorted)
    s+= u'encoding= %s\n'%(self.encoding)
    return s.decode(self.encoding,'ignore')

  def preview(self,n=None):
    """
    Show a preview of the publications (sorted).
    Optionally, only show the first n of them. If n is None, show all publications.
    """
    s=''
    nn= len(self.ListItems)
    if n != None:
      nn= min(n,nn)
    if not self.issorted:
      self.sort()
    for l in self.sortedList[:nn]:
      s+= '%s\n' %(self.get_item(l).preview())

    s=unicode(s,self.encoding,'ignore')
    return s

  def add_item(self, bib, key=None):
    be= bibitem.BibItem(bib,key)
    key= be.get_key()
    if key == None:
      sys.stderr.write('%s\nENTRY FAILED TO IMPORT: %s\n%s%s\n'%(80*'*',bib.get_field('_code',''),bib,80*'*'))
      return False

    if key in self.ListItems:
      if key == self.get_item(key).get_field('_code'): 
        sys.stderr.write('W: ENTRY ALREADY PRESENT: %s (%s)\n'%(key,bib['_code']))
        return False
      else:
        key = be.get_field('_code')

    self.bib[key]= be
    self.ListItems.append(key)
    self.sortedList.append(key)
    self.issorted=False
    return True

  def remove_item(self, key):
    if key in self.ListItems[:]:
      self.ListItems.remove(key)
      self.sortedList.remove(key)
      del(self.bib[key])
      return True
    else:
      return False

  def get_items(self):
    return self.bib.values()

  def get_item(self,key):
    return self.bib.get(key)

  def set_item(self,key,value):
    "Set the value of a given item (that already exists)"
    if key in self.ListItems:
      self.bib[key].update(value)
      return True
    else:
      return False

  def insertAbbrev(self, abbrev, value):
    if abbrev in self.abbrevDict:
      return False
    self.abbrevDict[abbrev] = value
    return True

  # resolve all abbreviations found in the value fields of all entries
  def resolve_abbrevs(self):
    self.keepAbbrevs=False
    for k in self.ListItems:
      self.get_item(k).resolve_abbrevs(self.abbrevDict)

  def List(self):
    return self.ListItems
  
  def sort(self, order=[], reverse=None):
    """
    Sort the entries according to the specified order
    """
    if order == []:
      order= self.sortorder
    else:
      self.sortorder= order
    if reverse == None:
      reverse= self.reverse
    else:
      self.reverse= reverse

    numericfields= ['year','volume','number','firstpage','lastpage']
    sortorder=order
    sortorder.append('key')
    s=[]
    for k in self.ListItems:
      oo=[]
      for o in sortorder:
        if o in numericfields : # For Numerical values we complete to the left with zeros
          oo.append(self.get_item(k).get(o,'').zfill(10))
        elif o == 'author':
          oo.append(self.get_item(k).get_authors_last())
        elif o == 'date': # Shorthand for ['year','month']
          oo.append(self.get_item(k).get_field('year','').zfill(10))
          oo.append(self.get_item(k).get_field('month',''))
        else:
          oo.append(self.get_item(k).get_field(o,'ZZZZ'))  # At the end if they have not field o
      s.append(oo)

    s.sort(reverse=reverse)
    self.issorted=True
    self.sortedList=[x[-1] for x in s]
    return self.sortedList


  def search(self, findstr, fields=[], caseSens=False, types='all'):
    """
    Search on the bibliography for findstr
    The result is a list with the keys of the items that match the search
    keys are the keys that we look
    types are on what kind of publication do we search (article, book,...)
    """
    result = []
    for f in self.sortedList:
      if types=='all' or self.get_item(f).get('_type') in types:
        if findstr == '*': found= True
        else:     found= self.get_item(f).search(findstr,fields,caseSens)
        if found:
          result.append(f)
    return result

#   def recreate_keys(self):
#     for b in self.sortedList[:]:
#       b1= self.get_item(b)
#       if b1 != None:
#         self.ListItems.remove(b)
#         key= b1.recreate_key()
#     self.sortedList= self.ListItems[:]
#     self.sort()

  def normalize(self):
    '''Make bibtex key tha same that internal key'''
    for b in self.sortedList:
      self.get_item(b).normalize()
    
	#############################################################3
	# import methods
	#############################################################3
  def load(self, fname):
    '''
    Load a biblist from file "fname" using the standard cPickle module.
    It can be used uncompressed or compressed with gzip or bzip
    '''
    try:
      fi= helper.openfile(fname,'rb');  c= pickle.load(fi);  helper.closefile(fi)
    except: 
      raise ValueError, 'Error loading data'
    try:    self.update(c)
    except: raise ValueError, 'Error updating data'
      
  def dump(self, fname, protocol=pickle.HIGHEST_PROTOCOL):
    '''
    Store the biblist in file "fname" using the standard cPickle module.
    It can be used uncompressed or compressed with gzip or bzip
    '''
    # if not '.dmp' in fname: fname='%s.dmp' %(fname)
    try:
      fo=helper.openfile(fname,'wb');  pickle.dump(self,fo,protocol=pickle.HIGHEST_PROTOCOL);
      helper.closefile(fo)
    except:
      raise ValueError, 'Error loading data'

  def import_bibtex(self, fname=None, normalize=True, ReplaceAbbrevs=True):
    """
    Import a bibliography (set of items) from a file
    If normalize the code (citekey) is overwritten with a standard key following our criteria
    """
    ncount=0
    st,db= bibitem.bibparse.parsefile(fname)

    if st != []:
      for k,v in st.iteritems():
        self.insertAbbrev(k, v)

    self.keepAbbrevs = not ReplaceAbbrevs
    if db != None:
      for k,v in db.iteritems():
        b1= bibitem.BibItem(bibitem.bibparse.replace_abbrevs(self.abbrevDict,dict(v)), normalize = normalize)
        key= b1.get_key()               # The key is generated
        if self.keepAbbrevs:  status= self.add_item(v, key)
        else:                 status= self.add_item(b1,key)
        if status:
          ncount+= 1
          if normalize:   self.get_item(key).normalize() # _code is put equal to key
    self.sort()
    return ncount

  def import_ads(self, fname, normalize=True):
    """
    Import a bibliography (set of items) from a file
    If normalize the code (citekey) is overwritten with a standard key following our criteria
    """
    ncount=0
    db= bibitem.adsparse.parsefile(fname)
    if db != None:
      for k,v in db.iteritems():
        status= self.add_item(v)
        if status:
          ncount+= 1
          if normalize:
            self.get_item(self.ListItems[-1]).normalize()
    self.sort()
    return ncount

##########################################################################################
                                     # export methods
##########################################################################################
  def set_default_styles(self):
    for item in self.get_items():
      item.set_default_styles()
      
  def output(self, fout=None, formato='short', verbose=True):
    """
    Export all entries to a fout file with default options. All strings are resolved.
    following formats are defined:
    short (default)
    long
    bibtex
    latex
    html
    xml
    """

    if verbose:
      print '# %d items to output' %(len(self.ListItems))
    fform = formato.lower()[0]
    if fform == 'b':  self.export_bibtex(fout)
    elif fform == 'l' or fform == 't': self.export_latex(fout)
    elif fform == 'h':  self.export_html(fout)
    elif fform == 'x':  self.export_xml(fout)
    else:
      fi= helper.openfile(fout,'w')
      if fform == 'f':   fi.write(str(self))
      else:  fi.write(self.preview().encode(self.encoding))
      helper.closefile(fi)


  def to_bibtex(self, indent=2, width=80):
    """
    Export all entries to a bibtex file. All strings are resolved.
    """
    if not self.issorted:      self.sort()

    s=''
    if self.keepAbbrevs:
      # Abbreviations with no standard abbreviations
      abbrevs={}
      std_abb= [x[0] for x in helper.standard_abbrev]
      for k in self.abbrevDict.keys()[:]:
        if k not in std_abb:
          abbrevs[k]= self.abbrevDict[k]

      for k in sorted(abbrevs):
        s+='@STRING{%s = "%s"}\n'%(k,self.abbrevDict[k])
      s+='\n\n'

    for l in self.sortedList:
      s+= '%s\n' %(self.get_item(l).to_bibtex(width= width))
      
    if not self.keepAbbrevs:      return s 
    else:      return helper.reg_defstrng.sub(r'\1\2',s)


  def export_bibtex(self, fname=None,indent=2, width=80):
    """
    Export a bibliography (set of items) to a file in bibtex format:
    """
    fi= helper.openfile(fname,'w');  fi.write(self.to_bibtex(width));  helper.closefile(fi)

  ##############################
  def to_latex(self,style={}, label = r'\item'):
    if not self.issorted:  self.sort()

    s=''
    for l in self.sortedList:
      if self.keepAbbrevs:
        bib= bibitem.BibItem(self.get_item(l)) # copy the item to resolve_abbrevs
        bib.resolve_abbrevs(self.abbrevDict)
        ss= bib.to_latex(style)
      else:
        ss= self.get_item(l).to_latex(style)
      s+= '%s %s\n' %(label, ss)
    return s

      
  def export_latex(self, fname=None, style={}, label = r'\item', head=None,tail=None):
    """
    Export a bibliography (set of items) to a file in latex format:
    """
    if head == None:
      head= r'''\documentclass[12pt]{article}
\newcommand{\authors}[1]{#1}
\usepackage{hyperref}
\begin{document}
\begin{enumerate}
'''
    if tail== None:
      tail= r'\end{enumerate} \end{document}'
    s= '%s\n%s\n%s\n'%(head, self.to_latex(style=style, label = label).encode('latex'), tail)
    fi= helper.openfile(fname,'w');  fi.write(s);  helper.closefile(fi)

  ##############################
  def to_html(self,style={}):
    if not self.issorted:
      self.sort()

    s=''
    for l in self.sortedList:
      tipo= self.get_item(l).get_field('_type','article')

      if self.keepAbbrevs:
        bib= bibitem.BibItem(self.get_item(l)) # copy the item to resolve_abbrevs
        bib.resolve_abbrevs(self.abbrevDict)
        ss= bib.to_html(style)
      else:
        ss= self.get_item(l).to_html(style)
      s+= '<li class="%s"> %s </li>\n' %(tipo, ss)
    return s

  def export_html(self, fname=None, style={}, head='',tail='', separate_css='biblio.css',encoding='utf-8'):
    """
    Export a bibliography (set of items) to a file in bibtex format: style is a dictionary
    (like in bibitem objects) where the values is a pair (open,close) to insert around the
    data.
    head and tail are html code to insert before and after the list of publications
    separate_css may have the
    """
    # default style
    css_style="""
.title a,
.title {font-weight: bold;	color :    #416DFF; }
ol.bibliography li{	margin-bottom:0.5em;}
.journal {  font-style: italic;}
.book .series {  font-style: italic;}
.journal:after {content:" ";}
.series:after {content:" ";}
li.article .publisher {display:none;}
.publisher:before {content:" (";}
.publisher:after {content:") ";}
.year:before {content:" (";}
.year:after {content:").";}
.authors {font-weight:bol; display:list;}
.authors:after {content:". ";}
.volume { font-weight: bold;}
.book .volume: before { content: "Vol. ";}
.number:before {content:":";}
.button {display:inline; border: 3px ridge;line-height:2.2em;margin: 0pt 10pt 0pt 0pt;padding:1pt;}
.masterthesis{content:"Master Thesis"}
.phdthesis{content:"Phd Thesis"}
div.abstracts {display: inline; font-weight: bold; text-decoration : none;  border: 3px ridge;}
div.abstract {display: none;padding: 0em 1% 0em 1%; border: 3px double rgb(130,100,110); text-align: justify;} 
    """

    if helper.is_string_like(separate_css):
      fi=helper.openfile(separate_css,'w');  fi.write(css_style);  helper.closefile(fi)
      name= os.path.commonprefix([os.path.dirname(fname),os.path.dirname(separate_css)])
      print os.path.basename(separate_css)
      name= os.path.join(name,separate_css[len(name):])
      css='  <link title="new" rel="stylesheet" href="' + name + '" type="text/css">'
    else:
      css= '<style type="text/css">' +css_style + '</style>'

    if head=='':
      head='''
    <html>
    <head>
    <meta http-equiv="Content-Type" content="text/html; charset='''+encoding.upper()+'''">
    ''' +  css + '''
    <title>Publicaciones</title>
    <script language="JavaScript" type="text/javascript">
    //<![CDATA[
    function toggle(thisid) {
    var thislayer=document.getElementById(thisid);
    if (thislayer.style.display == 'block') {
    thislayer.style.display='none';
    } else {
    thislayer.style.display='block';}
    }
    //]]>
    </script>
    </head>
    <body>
    <h2>Publicaciones</h2>
    <ol class="bibliography">
    '''
    if tail == '':
      tail="""
      </ol>
      </body>
      </html>
      """
      
    s= head + self.to_html(style=style) + tail
    fi= helper.openfile(fname,'w');  fi.write(s.encode(encoding,'xmlcharrefreplace'))
    helper.closefile(fi)

  ##############################
  def to_xml(self, prefix='', indent=2):
    if not self.issorted:      self.sort()

    s=''
    for l in self.sortedList:
      if self.keepAbbrevs:
        bib=bibitem.BibItem(self.get_item(l)) # copy the item to resolve_abbrevs
        bib.resolve_abbrevs(self.abbrevDict)
        ss= bib.to_xml(p=prefix,indent=indent)
      else:
        ss= self.get_item(l).to_xml(p=prefix,indent=indent)
      
      s+= '%s' %(ss)
    return s
      
  def export_xml(self, fname=None, prefix='', head='', tail='', indent=2):
    """
    Export a bibliography (set of items) to a file in xml format:
    A prefix may be added to account for a namespace. But if added both head and tail
    should take it into account to make it a valid xml document
    """
    if head == '':
      head='''<?xml version="1.0" encoding="utf-8"?>
  <''' + prefix +'''bibliography>
'''
    if tail == '':
      tail="\n</"+prefix+"bibliography>"
      
    s= head + self.to_xml(prefix=prefix) + tail
    fi= helper.openfile(fname,'w');  fi.write(s.encode('utf-8','xmlcharrefreplace'))
    helper.closefile(fi)
##########################################################################################
##########################################################################################
    
def test():
  if sys.argv[1:]:
    filepath= sys.argv[1]
  else:
    print "No input file"
    print "USAGE:  "+sys.argv[0]+ " FILE.bib\n\n  It will output the XML file: FILE.xml"
    sys.exit(2)

  biblio= BibList()
  if filepath.find('.bib') != -1:
    nitems= biblio.import_bibtex(filepath,False,False)
    print '%s\nFrom BibTeX file: %s'%(80*"*",filepath)
  elif filepath.find('.ads') != -1:
    nitems= biblio.import_ads(filepath,True)
    print '%s\nFrom ADS file: %s'%(70*"*",filepath)
  print '%d items ingresados\n'%(nitems)
  print 'Rodrig en los siguientes items:',biblio.search('Rodrig')
  print 20*'='
  print 'Items Ordenados por cite: %s' %(biblio.sort(['key']))
  print 20*'*'
  print 'Items Ordenados por Apellido de Autores: %s' %(biblio.sort(['author']))
  print 20*'*'
  print 'Items Ordenados por Fecha: %s' %(biblio.sort(['date']))
  print 20*'*'
  nn=5
  print 'Preview (At most %d items):'%(nn)
  print 20*'*'
  print biblio.preview(nn)
  print 'Preview with LaTeX symbols (At most First %d items):'%(nn)
  print 20*'*'
  print biblio.preview(nn).encode('latex')
  biblio.export_bibtex('tempo.bib',4)
  biblio.export_html('tempo.html')
  biblio.export_xml('tempo.xml',prefix='')

def main():
  test()


if __name__ == "__main__": main()



#!/usr/bin/python
#
# File: setup.py
#
# This file is part of the biblio-py project
#
# License:
# 
# Copyright (C) 2009 Juan Fiol This is free software.
# 
# You may redistribute copies of it under the terms of the GNU General Public License
# version 2 or later.  There is NO WARRANTY, to the extent permitted by law.
#
# Written by Juan Fiol <juanfiol@gmail.com>
import os
import glob
from distutils.core import setup
# import distutils

VERSION= '0.5.8'
def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = (
    read('README')
    + '\n' +
    'Change history\n'
    '**************\n'
    + '\n' + 
    read('CHANGES')
    + '\n'
    )

packages = ["yapbib","query_ads"]
scripts= glob.glob('scripts/*.py')

# do the distutils setup
setup(name="biblio-py",
      version = VERSION,
      description = "Package to manage bibliography files",
      long_description = long_description,
      license = "GPLv2",
      url = "http://fisica.cab.cnea.gov.ar/colisiones/staff/fiol/biblio-py.html",
      download_url="http://fisica.cab.cnea.gov.ar/colisiones/staff/fiol/files/biblio-py-%s.tar.gz"%(VERSION),
      keywords= "bibliography, bibtex, converter, html, xml, latex, parser",
      author = "Juan Fiol",
      author_email = "juanfiol@gmail.com",
      packages = packages,
      scripts = scripts,
      )




#!/usr/bin/env python
'''
Class storing a bibligraphic item (paper, book, manual, ...)

Required fields: _code, _type, author, year

Internally authors are represented as a list [first author, second author, ...]
where each author is represented itself as a list:
                                      [von, Last, First, Jr]
following the terminology of BibTeX.
Note that BibTeX would use DIFFERENTLY:  von Last, Jr, First

There are two field related to the journal: journal and journal_abbrev 

'''
import sys
import os
import textwrap
import codecs

import bibparse
import adsparse
import helper
import latex
latex.register()


# Index used for internally for each part of a name
A_VON=0; A_LAST=1; A_FIRST=2; A_JR=3
##########################################################################################
class BibItem(dict):
  """
  Class that store a bibliography item
  """

  def __init__(self,bib={},key=None,normalize=False):
    """
    Initialize the object. 
    """
    self.key=''
    self.html_style={}
    self.latex_style={}
    if bib=={}:
      self.update({})
      self.key=None
      self.set_default_styles()
    else:
      self.set(bib, key)
      if normalize: self.normalize()
    self.encoding='utf8'

  def set(self, b, key=None):
    """
    Set the values of the object to those given by the dictionary.
    If b is not a valid dictionary it will return an empty object
    """
    t= self._verify_entry(b)
    if t:
      # Copy the dictionary
      self.update(b)

      if key != None:
        self.key= key
      else:
        try: 
          if b.get_key() != None:  self.key= b.get_key()  # b is a BibItem object but has not key
          else: self.key= self.create_entrycode()
        except:  # b is a dictionary
          self.key= self.create_entrycode()

      if self.key == '':
        self.key=None
      elif self.get_field('_code','') == '':
        self.normalize()
      try:
        self.set_styles(html=b.html_style, latex= b.latex_style)
      except:
        self.set_default_styles()
    else:
      self.key=None

  def resolve_abbrevs(self, strs={}):
    bibparse.replace_abbrevs(strs,self)
  
  def _verify_entry(self,b):
    '''Verify that the entry is valid'''
    for f in helper.minimalfields:
      if not b.has_key(f):
        return False

    if b.has_key('journal'):
      if not b.has_key('journal_abbrev'):
      # Create one abbrev
        journal, abbrev= helper.identify_some_journals(b)
        b['journal_abbrev']= abbrev
        b['journal']= journal
        return True
    return True

   
  def normalize(self):
    self['_code']= self.key

  def set_default_styles(self):
    default_html_style={
      '_type'      :None,
      'author'     :(r'<span class="authors">',r'</span> '),
      'title'      :(r'<span class="title">',r'</span> '),             
      'journal'    :(r'<span class="journal">',r'</span> '),
      'school'     :(r'<span class="journal">',r'</span> '),
      'volume'     :(r'<span class="volume">',r'</span> '),
      'number'     :(r'<span class="number">',r'</span> '),
      'year'       :(r'<span class="year">',r'</span> '),
      'publisher'  :(r'<span class="publisher">',r'</span> '),
      'series'     :(r'<span class="series">',r'</span> '),
      'firstpage'  :(r' ',r''),
      'lastpage'   :(r'-',r''),
      'booktitle'  :(r'<span class="journal">',r'</span> '),             
      'chapter'    :(r'<span class="volume">',r'</span> '),
      'institution':None,
      'organization':None,
      'address'    :None,
      'edition'    :(r'<span class="volume">',r'</span> '),
      'issn'       :None,
      'isbn'       :None,
      'note'       :None,
      'annote'     :None,
      'month'      :None,
      'keywords'   :None,
      '_code':(r' <a class="button" title="Download Local Copy" href="deposito/',r'.pdf">L-C</a>'),
      'url'        :(r'  <a class="button" title="External Link" href="',r'">E-L</a>'),
      'doi'        :(r'  <a class="button" title="Get from source" href="http://dx.doi.org/',r'">DOI</a>'),
      'abstract'   :(r'<div class="abstracts button" onClick="toggle('+"'"+self.key.lower()+"'"+')">ABS</div>\n<div class="abstract" id="'+ self.key.lower()+'">',r'</div>')
      }
    default_latex_style={
      'author'     : (r'\authors{',r'}. '),
      'title'      : (r' \textbf{',r'} '),
      'journal'    : (r' \emph{',r'} '),
      'booktitle'  : (r' \emph{',r'}, '),
      'volume'     : (r' \textbf{',r'}'),
      'year'       : (r' ',r'.'),
      'firstpage'  : (r' ',r''),
      'lastpage'   : (r'-',r','),
      'number'     : (':',''),
      'school'     : ('',''),
      'url'        : (r' \url{',r'}'),
      'doi'        : (r' \url{',r'}'),
      'abstract'   : None,
      '_code': None,
      }

    self.html_style =  default_html_style
    self.latex_style= default_latex_style

  def set_styles(self,html={},latex={}):
    # Output styles
    if html != {}:
      self.html_style.update(html)
    if latex != {}:
      self.latex_style.update(latex)
  
  def get_key(self):
    return self.key

  def get_fields(self):
    return sorted(self.keys())
  

  def __str__(self):
    s= "%15s: %s\n" %("Type", self.get_field('_type'))
    s+="%15s: %s\n" %("citekey", self.get_field('_code'))
    s+="%15s: %s\n" %("internal key", self.get_key())

    for k in self.keys():
      if k.startswith('_'):
				continue;
      s+= "%15s: %s\n" %(k, helper.reg_defstrng.sub(r'\1\2',self.get_field(k,'')))
    try: 
      return unicode(s,self.encoding,'ignore')
    except:
      return s
  
  def preview(self):
    """
    Preview some information on the item. It does not print correctly when missing fields
    """
    s = "%22s: %s, " %(self.key, self.get_authors(Initial=True,smart=True))
    s+= "%s %s, " %(self.get_field('journal_abbrev',''), self.get_field('volume','') )
    s+= "%s (%s).\n" %(self.get_field('pages'), self.get_field('year','') )

    return s.encode(self.encoding,'ignore')

  def get_listnames_last(self, who='author', strict=False):
    """
    Retrieve the authors Last names and return them as a list
    Default bib.get_listnames_last() returns: [von last1, von last2, ...]
    """
    who= who.lower()
    if who not in ['author','editor']:
      raise AttributeError, "who must be author or editor, not %s"%(who)

    if who in self.keys():
      if strict: # Return Last names
        return [x[A_LAST] for x in self[who]]
      else: # Return Full last names ('von Last')
        return ['%s %s'%(x[A_VON],x[A_LAST]) for x in self[who]]
    else:
      return []

  def get_authors_last(self, separator=', '):
    "Returns a string with the last names of the authors separated by a comma"
    return separator.join(self.get_listnames_last(who='author',strict=False))

  def _format_one_author(self,auth, format=0, Initial=False):
    """Returns the author as:
    format= 0  =>   F[irst|.] M[iddle|.] von Last
    format= 1  =>   von Last, F[irst|.] M[iddle|.]
    If Initial is True, only the initials of first names are printed
    """
    
    first=''
    try:
      aa=auth[A_FIRST].split()
    except:
      print auth
      sys.exit()
    for d in aa:
      if Initial or len(d)==1:
        first += d[0]+'.'
      else:
        first+= d.strip()
      first+=' '
    first= first.strip()

    if format==1:
      autor= '%s %s'%(auth[A_VON], auth[A_LAST])
      if first!= '':
        autor +=', %s' %(first)
        if auth[A_JR].strip() != '':
          autor +=', %s' %(auth[A_JR])
    else: # Default format
      autor= ' '.join([first,auth[A_VON], auth[A_LAST]])
    return autor.replace('  ',' ')

  def get_authorsList(self, format=0, Initial=False, who='author'):
    """
    Returns a list of authors where each author is in the form:
    format= 0  =>   F[irst|.] M[iddle|.] von Last
    format= 1  =>   von Last, F[irst|.] M[iddle|.]
    """
    if self.has_key(who):
      return map(self._format_one_author,self[who],len(self[who])*[format],len(self[who])*[Initial])
    else:
      return [[]]

  def get_first_author(self):
    return self.get_authorsList(format=1,Initial=True)[0]
    
  def get_editors(self):
    aa= self.get_authorsList(format=0,Initial=False)
    if len(aa)== 1:
      return aa[0]
    else: 
      return ', '.join(aa[:-1])+ ' and ' + aa[-1]
    
  def get_authors(self, Initial=False, smart=False):
    """
    Returns a string with the authors in the form:
    author_1, author_2, author_3, ... author_n-1 and author_n
    and each author has the form: F[irst|.] M[iddle|.] [von] Last
    """
    MAX_AUTHORS=5
    aa= self.get_authorsList(format=0,Initial=Initial)
    if len(aa)== 1:
      return aa[0]
    elif len(aa) > 1:
      if smart and len(aa) > MAX_AUTHORS:
        return ', '.join(aa[:MAX_AUTHORS])+ ' et al'
      else:
        return ', '.join(aa[:-1])+ ' and ' + aa[-1]

  def get_affiliation(self):
    return ';'.join(self.get('affiliation',[]))


  def get_type(self):
    return self.get_field('_type','article')
  
  def create_entrycode(self, _create_func_=None):
    """  Creates a 'hopefully unique' key    """
    if _create_func_ == None:
      return bibparse.create_entrycode(self)
    else:
      return _create_func_(self)

  def get_field(self,field,d=None):
    if field == 'key':
      return self.key
    elif field == 'author':
      return self.get_authors()
    elif field == 'editor':
      return self.get_editors()
    elif field == 'affiliation':
      return self.get_affiliation()
    elif field == 'pages':
      return self.getpages()
    else:
      return self.get(field,d)
    
  def getpages(self):
    s=''
    if self.has_key('firstpage'):
      s+= self['firstpage'].strip()
      if self.has_key('lastpage'):
        p= self['lastpage'].strip()
        if p != '':
          s+= '-%s' % (p)
    return s

  def to_bibtex(self, indent=2, width=80):
    """
    Format an entry as a bibtex item and returns it as a string, the argument wrap is the length of the lines in output
    """
    
    initial_indent= indent*' '
    # Indent the values of the fields
    wrap= textwrap.TextWrapper(width=width,subsequent_indent= 3*initial_indent,break_long_words=False)
    s= '@%s{%s,\n' %(self['_type'].upper(),self['_code'])

    # Add list of authors
    for f in ['author','editor']:
      if self.has_key(f):
        autores= ' and '.join(self.get_authorsList(format=1,who=f))
        wrap.initial_indent=initial_indent
        s+= wrap.fill(f + ' = {' + autores +'},') + '\n'

    # put braces around upper case in titles
    textualfields = helper.textualfields[:]
    braces_field = ['title']
    for kk in braces_field: 
      textualfields.remove(kk)
      if self.has_key(kk): 
        tit= helper.removebraces(self[kk])
        f = tit[0]
        for  c in tit[1:]:
          if c.isupper():
            f = f + '{' + c + '}'
          else: f = f + c
        s += wrap.fill('%s = {%s},' % (kk , f))+'\n'

    # Some fields that are copied literally
    for kk in textualfields:
      if self.has_key(kk) and not kk.startswith('_'):
        if kk in helper.nowrapfields:  # Not wrap
          s+= '%s%s = {%s}, \n' % (initial_indent,kk , self[kk])
        else:
          s+= wrap.fill('%s = {%s},' % (kk , self[kk]))+'\n'
    # Pages field
    p= self.get_field('pages')
    if p != '':
      s+= '%spages = {%s},\n' %(initial_indent,p)
    # Close the bibitem       
    s+= '}\n'
    s= s.encode('latex','ignore')  # Convert to latex some characters using encoding
  #   s= handle_math(s,orden=1)      # Extra handling of math expressions (very simple)
    return s

  def to_xml(self,p='',indent=2):
    """
    Converts the item to xml format. The prefix is added to each entry
    """
    sp= indent
    spc=indent*' '
    s='%s<%sentry id="%s">\n' %(sp*spc,p,self.get_field('_code',''))
    sp += 1
    s+='%s<%s%s>\n' %(sp*spc,p,self.get('_type',''))

    for k,e in self.iteritems():
      if k == 'author':
        sp+=1
        space=sp*spc+'\n'
        v= space.join(['%s<%sauthor>%s</%sauthor>'%(sp*spc,p,x,p) for x in self.get_authorsList()])
        v= helper.removebraces(v)
        v= helper.replace_tags(v,'other')
        sp-=1
        s+= '%s<%s%s>\n%s\n%s</%s%s>\n' %(sp*spc,p,'authors',v,sp*spc,p,'authors')
      else:
        if helper.is_string_like(e):
          v= helper.replace_tags(e,'xml')
          v= helper.handle_math(v)
        if k=='title':
          v=helper.capitalizestring(v)
          v= helper.removebraces(v)
          v= helper.replace_tags(v,'other')
        s+= '%s<%s%s>%s</%s%s>\n' %(sp*spc,p,k,v,p,k)

    sp-=1
    s+= '%s</%s%s>\n' %(sp*spc,p,self.get('_type',''))
    s+= '%s</%sentry>\n' %(sp*spc,p)
    return s

  def to_html(self,style={}):
    """
    Converts the item to html format with the given style The style is a pair (before,
    after) surrounding the corresponding field (except for authors)
    """
    # textfields= ['journal','booktitle','series','school','volume','number','publisher','firstpage','lastpage','year','_code','url','doi','abstract']
    textualfields = ['journal', 'volume','number', 'month', 'booktitle', 'chapter', 'address', 'edition', 'howpublished','school', 'institution', 'organization', 'publisher', 'series','firstpage','lastpage',
                     'note', 'crossref', 'issn','isbn', 'year', 'keywords' ,'annote', '_code', 'doi',  'url',  'abstract']

    st= dict(self.html_style)
    st.update(style)
    tipo= self.get('_type','article')
    title= helper.handle_math(self.get_field('title',''))
    title= helper.removebraces(title)
    if 'mastersthesis' == tipo:  title= 'Master Thesis. ' + title
    elif 'phdthesis' == tipo:    title= 'Phd Thesis. ' + title

    s= ''
    if title != '' and st.get('title',['','']) !=  None:
      s+= title.strip().join(st.get('title',['','']))

    if self.has_key('author') and st.get('author')!= None:
      form_aut=''
      list_aut= self.get_authorsList()
      if len(list_aut) > 10:
        list_aut=list_aut[:10]
        list_aut.append('<i>et al</i>')
      for a in list_aut:
        form_aut += '<span class="author">'+ a +'</span>, '
      form_aut= form_aut[:-2]
      s+= form_aut.join(st.get('author',['','']))
    
    for field in textualfields:
      if self.get_field(field,'') != '' and st.get(field,(' ',' ')) != None:
        value= helper.handle_math(self.get_field(field,'').strip())
        value=bibparse.helper.removebraces(value)
        s+= value.join(st.get(field,[' ',' ']))

    if st.get('_type',['','']) !=  None:
      s= s.strip().join(st.get('_type',['','']))

    s= s.decode('latex','replace')  # Convert from latex some characters using encoding
    return unicode(s)

  def to_latex(self,style={}):
    """
    As its name indicates, it converts bibtex data to a latex bibitem
    """

    textfields= ['journal','booktitle','school','volume','number','firstpage','lastpage','year','url','doi','abstract']

    st=dict(self.latex_style)
    st.update(style)
    s=''
    
    if st.get('_code') !=  None:
      s+= self.get('_code').join(st['_code'])

    if self.get('title','') != '' and st.get('title') !=  None:
      s+= self.get('title').join(st['title'])

    if self.get('author','') != '' and st.get('author') != None:
      s+= ', '.join(self.get_authorsList()).join(st['author'])

    for field in textfields:
      if self.get_field(field,'') != '' and st.get(field) != None:
        s+= self.get_field(field,'').strip().join(st[field])
    # return s.encode('latex')
    return s

  def display(self, fpp=sys.stdout):
    if type(fpp)==type(''):
      fp= codecs.open(fpp,'w', encoding=self.encoding)
    else:
      fp=fpp
    s= str(self)
    fp.write(s)


	#############################################################3
	# import methods
	#############################################################3
  def from_bibtex(self,source):
    """
    Reads an item in bibtex form from a string 
    """
    try:
      source+' '
    except:
      raise TypeError, 'source must be a string'
    st,entry= bibparse.parseentry(source)
    if entry != None:
      self.set(entry)

  def from_ads(self,source):
    """
    Reads an item in bibtex form from a string 
    """
    try:
      source+' '
    except:
      raise TypeError, 'source must be a string'
    entry= adsparse.parseentry(source)
    if entry != None:
      self.set(entry)


	#############################################################3
	# matching methods
	#############################################################3

  def search(self, findstr, fields=[], caseSens=False):
    if findstr=='*': return True
    if fields==[]:   # Busca en todos los campos
      fields=self.get_fields()

    if caseSens:    s=findstr
    else:           s=findstr.lower()

    for f in fields:
      if f not in self.get_fields():
        continue
      v= self.get_field(f)
      if not caseSens:
        try:      v= v.lower()
        except:   pass
      if s in v:  return True

    # Search for string in key
    if 'key' in fields:
      if caseSens: return s in self.key
      else: return s in self.key.lower()
    else:
      return False
  
  def matchAuthorList(self,it):
    """ Test whether two publications have the same authors """
    # First we match the last (von Last) names
    a1s=self.get_listnames_last('author',False)
    a2s=it.get_listnames_last('author',False)
    for a1,a2 in zip(a1s,a2s):
      if a1.strip().lower() != a2.strip().lower():
        return False

    # In case it did worked, we check the initials
    a1s= [x[A_LAST][0] for x in self['author']]
    a2s= [x[A_LAST][0] for x in it['author']]
    for a1,a2 in zip(a1s,a2s):
      if a1 != a2:
        return False

    # Some other conditions
    # ....

    # Else they are equal
    return True

  def compare(self, it):
    """ Compare if two items are in fact the same """

    # If the doi is the same, we are done
    if self.get('doi','0') == self.get('doi','1'):
      return True

    # Check Journal and page, that should be enough (almost always)
    if self('journal_abbrev','0')== it.get('journal_abbrev','1'):
      # Same journal AND either first page or volume coincide we assume is the same
      if self.get('firstpage','-1') == it.get('firstpage','-2'):
        return True
      if self.get('volume','-1') == it.get('volume','-2'):
        return True

    if self.matchAuthorList(it):
      # If authors AND either title or first page coincide we assume is the same
      if self.get('title','0') == it.get('title','1'): 
        return True
      elif self.get('firstpage','-1') == it.get('firstpage','-2'):
        return True

    # Otherwise they are different
    return False


def test():
  """
  Test the class and its methods
  """
  
  css="""
.title a,
.title {  font-weight: bold;	color: rgb(20,20,20) ;}
ol.bibliography li{	margin-bottom:0.5em;}
.journal {  font-style: italic;}
.journal:after {content:", ";}
.publisher:before {content:" (";}
.publisher:after {content:") ";}
.series:after {content:", ";}
.year:before {content:" (";}
.year:after {content:").";}
.authors {display:block;}
.volume { font-weight: bold;}
.button {display:inline; border: 3px ridge;line-height:2.2em;margin: 0pt 10pt 0pt 0pt;padding:1pt;}
.masterthesis{content:"Master Thesis"}
.phdthesis{content:"Phd Thesis"}
div.abstracts {display: inline; font-weight: bold; text-decoration : none;  border: 3px ridge;}
div.abstract {display: none;padding: 0em 1% 0em 1%; border: 3px double rgb(130,100,110); text-align: justify;} 
"""
  hhead='''
  <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"> <html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
  <head>  <title>Paper</title>  <meta http-equiv="Content-Type"  content="text/html; charset=UTF-8" /> <meta name="author" content="J. Fiol" /> <meta name="description" content="A list of publications entries, designed for Colisiones Atomicas, CAB." /> <style type="text/css">
'''+css+'''	</style>  <script language="JavaScript" type="text/javascript">
      //<![CDATA[
      function toggle(thisid) { var thislayer=document.getElementById(thisid); if (thislayer.style.display == 'block') { thislayer.style.display='none';} else { thislayer.style.display='block';}}//]]> </script></head> <body id="page-body">
  '''
  hfoot=""" </body></html> """

  b=BibItem({'_type':'article','author':[['della', 'Picca','Renata',''],['',r'Mart{\'{\i}}nez','R. O.',''],['','Fiol','J.',''],['','Macri','P.','']],'year':'2008','firstpage':'402', 'lastpage':'406', 'abstract':u'We employ different theoretical models, both classical and quantum-mechanical, to explore the recoil-ion momentum distribution in positron atom collisions. We pay special attention to the vicinity of the kinematical threshold between ionization and positronium formation. We demonstrate that it is intertwined by dynamical constraints to the formation of highly excited and low-lying continuum electron positron states. Finally we discuss how the study of recoil- ion momentum distribution, which is characteristic of a reaction microscopy technique, might represent an alternative approach to the standard spectroscopy of electrons and positrons.',  'title' : u'Threshold effects in the ionization of atoms by positron impact',
   '_code':'Fiol07NIMB',
  'journal' : u'Nuclear Instruments and Methods in Physics Research B',
  'year' : '2008',
  'volume' : '266',
  'url' : u'http://adsabs.harvard.edu/abs/2008NIMPB.266..402B',
  'doi' : '10.1016/j.nimb.2007.12.040',
})

  btest=r"""
  @ARTICLE{Fiol07NIMB,
  author = {Renata della Picca and R. O. Mart{\'{\i}}nez and J. Fiol and P.
  Macri},
  title = {Threshold effects in the ionization of atoms by positron impact},
  journal = {Nuclear Instruments and Methods in Physics Research B},
  year = {2008},
  volume = {266},
  doi = {10.1016/j.nimb.2007.12.040},
  url = {http://adsabs.harvard.edu/abs/2008NIMPB.266..402B},
  abstract = {We employ different theoretical models, both classical and
      quantum-mechanical, to explore the recoil-ion momentum distribution in
      positron atom collisions. We pay special attention to the vicinity of the
      kinematical threshold between ionization and positronium formation. We
      demonstrate that it is intertwined by dynamical constraints to the
      formation of highly excited and low-lying continuum electron positron
      states. Finally we discuss how the study of recoil- ion momentum
      distribution, which is characteristic of a reaction microscopy technique,
      might represent an alternative approach to the standard spectroscopy of
      electrons and positrons.},
  pages = {402-406},
}
"""
  atest=u"""
%R 2008JPhB...41n5204M
%T Transfer ionization and total electron emission for 100 keV amu<SUP>-1</SUP> 
He<SUP>2+</SUP> colliding on He and H<SUB>2</SUB>
%A Mart\xednez, S.; Bernardi, G.; Focke, P.; Su\xe1rez, S.; Fregenal, D. 
%F Centro At\xf3mico Bariloche and Instituto BalseiroComisi\xf3n Nacional de Energ\xeda 
At\xf3mica and Universidad Nacional de Cuyo, Argentina., 8400 S C de Bariloche, 
R\xedo Negro, Argentina <EMAIL>bernardi@cab.cnea.gov.ar</EMAIL>
%J Journal of Physics B: Atomic, Molecular, and Optical Physics, Volume 41, 
Issue 14, pp. 145204 (2008).
%V 41
%D 7/2008 
%P 5204
%G IOP
%I ABSTRACT: Abstract;
   EJOURNAL: Electronic On-line Article (HTML);
   REFERENCES: References in the Article;
   AR: Also-Read Articles;
%U http://adsabs.harvard.edu/abs/2008JPhB...41n5204M
%B We have measured electron emission for transfer ionization (TI) and 
total electron emission (TEE, all emission processes) for 100 keV 
amu<SUP>-1</SUP> He<SUP>2+</SUP> on He and H<SUB>2</SUB> targets. Double 
differential cross sections have been obtained for emission angles 
\u03b8 = 0\xb0, 20\xb0 and 45\xb0, and electron energies ranging 
from 2 to 300 eV. Pure ionization, mainly due to single ionization, 
dominates the low-energy electron emission. The main observed structure 
in the electron spectra, a cusp centred at \u03b8 = 0\xb0 and at a 
speed equal to that of the incident projectile, presents an asymmetric 
shape. This is in contrast to the symmetric shape observed by us at 25 
keV amu<SUP>-1</SUP> for the same collision systems, suggesting a change 
in the cusp formation mechanism for TI within this energy range.
%Y DOI: 10.1088/0953-4075/41/14/145204
"""
  f=codecs.open('bibit.html','w',encoding='utf8'); f.write(hhead+b.to_html()+ hfoot); f.close()
  f=codecs.open('bibit.bib','w'); f.write(b.to_bibtex(width=80)); f.close()
  f=codecs.open('bibit.tex','w'); f.write(textwrap.fill(b.to_latex(style={'_code':(r'[',r'] ')}),width=120) ); f.close()
  f=codecs.open('bibit.xml','w'); f.write(b.to_xml()); f.close()

  # Other item copied from b
  print 80*'*'
  print 'Define one entry...'
  print 30*'*'
  b.display()   # Display to std output

  c=BibItem(b,normalize=True)
  print '\n',80*'*','\nc=BibItem(b,normalize=True)',
  print '\n',30*'*'
  print '\nc='
  print c
  print 80*'*'
  print 'Testing some Methods...'
  print 30*'*'
  print '\nc.get_key()=',c.get_key()
  print '\nc.Fields()=',c.get_fields()
  print '\nc is b? ->', c is b,',      c == b? ->', c == b

  print "c.search('colisions'): ",c.search('colisions')
  print "c.search('collisions'): ",c.search('collisions')
  print "c.search('Fiol'): ",c.search('Fiol')
  print "c.search('Nuclear'): ",c.search('Nuclear')
  print "c.search('Nuclear',['author','year']): ",c.search('Nuclear',['author','year'])
  print "c.search('Nuclear',['author','year','journal']): ",c.search('Nuclear',['author','year','journal'])

  d= BibItem()
  d.from_bibtex(btest)
  print 80*'*'
  print 'Entry from a source in BIBTEX format'
  print 30*'*'
  d.display()
  print 80*'*'
  e= BibItem()
  e.from_ads(atest)
  print 'Entry from a source in ADS PORTABLE format'
  print 30*'*'  
  e.display()

  # Display some message
  mensaje=u'Se han escrito ejemplos de una publicacion en formatos latex, bibtex, xml y html en los archivos bibit.tex, bibit.bib, bibit.xml y bibit.html'

  print '%s\n%s\n%s' %(80*'*', textwrap.fill(mensaje,width=80),80*'*')


def main():
  test()

if __name__ == "__main__": main()




# ##########################################################################################
#        ################################ Variables  ################################
# # list of additional fields, ignored by the standard BibTeX styles
# ign = ('crossref', 'code', 'url', 'annote', 'abstract');

# # lists of required and optional fields for each reference type

# required_fields = {
#   'article' :		['Author', 'Title', 'Journal', 'Year'],
#   'book' :		['Author', 'Title', 'Publisher', 'Year']
#   'booklet' :		['Title'],
#   'inbook' :		['Author', 'Title', 'Chapter', 'Pages', 
#   				'Publisher', 'Year'],
#   'incollection' :	['Author', 'Title', 'Booktitle', 'Publisher', 'Year'],
#   'inproceedings' :	['Author', 'Title', 'Booktitle', 'Year'],
#   'manual' :		['Title'],
#   'misc' : 		[],
#   'mastersthesis' :	['Author', 'Title', 'School', 'Year'],
#   'phdthesis' :		['Author', 'Title', 'School', 'Year'],
#   'proceedings' :	['Title', 'Year'],
#   'techreport' :	['Author', 'Title', 'Institution', 'Year'],
#   'unpublished' :	['Author', 'Title', 'Note']
# };

# opt_fields = {
#   'article' :		['Volume', 'Number', 'Pages', 'Month', 'Note'],
#   'book' :		['Editor', 'Volume', 'Number', 'Series', 'Address',
#   				'Edition', 'Month', 'Note'],
#   'booklet' :		['Author', 'Howpublished', 'Address', 'Month', 'Year',
#   				'Note'],
#   'inbook' :		['Editor', 'Volume', 'Series', 'Address', 'Edition',
#   				'Month', 'Note'],
#   'incollection' :	['Editor', 'Volume', 'Number', 'Series', 'Type', 
#   				'Chapter'  'Pages', 'Address', 'Edition',
# 				'Month', 'Note'],
#   'inproceedings' :	['Editor', 'Pages', 'Organization', 'Publisher', 
#   				'Address', 'Month', 'Note'],
#   'manual' :		['Author', 'Organization', 'Address', 'Edition',
#   				'Month', 'Year', 'Note'],
#   'misc' :		['Title', 'Author', 'Howpublished', 'Month', 'Year',
#   				'Note'],
#   'mastersthesis' :	['Address', 'Month', 'Note'],
#   'phdthesis' :		['Address', 'Month', 'Note'],
#   'proceedings' :	['Editor', 'Publisher', 'Organization', 'Address', 
#   				'Month', 'Note'],
#   'techreport' :	['Type', 'Number', 'Address', 'Month', 'Note'],
#   'unpublished' :	['Month', 'Year']
# };

# ################################################################################

